#!/usr/bin/env python
#   This file is part of nexdatas - Tango Server for NeXus data writer
#
#    Copyright (C) 2012-2017 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
# \package nxsconfigtool nexdatas
# \file ItemCommands.py
# user commands of GUI application

""" Component Designer commands """

from PyQt5.QtWidgets import QMessageBox, QUndoCommand

from . import DataSource
from .Component import Component
from .ComponentModel import ComponentModel
from .EditCommands import (DataSourceCut, DataSourceCopy, DataSourcePaste)

import logging
# message logger
logger = logging.getLogger("nxsdesigner")


# Abstract Command which helps in defing commands related to
#  Component item operations
class ComponentItemCommand(QUndoCommand):

    # constructor
    # \param receiver command receiver
    # \param parent command parent
    def __init__(self, receiver, parent=None):
        QUndoCommand.__init__(self, parent)
        # main window
        self.receiver = receiver
        self._cp = None
        self._oldstate = None
        self._index = None
        self._newstate = None
        self._subwindow = None

    # helps to construct the execute component item command as a pre-executor
    # \brief It stores the old states of the current component
    def preExecute(self):
        if self._cp is None:
            self.receiver.updateComponentListItem()
            self._cp = self.receiver.componentList.currentListElement()
        if self._cp is not None:
            if self._oldstate is None and hasattr(self._cp, "instance") \
                    and hasattr(self._cp.instance, "setState"):
                self._oldstate = self._cp.instance.getState()
                self._index = self._cp.instance.currentIndex()

            else:
                QMessageBox.warning(
                    self.receiver, "Component not created",
                    "Please edit one of the components")

        else:
            QMessageBox.warning(
                self.receiver, "Component not selected",
                "Please select one of the components")

    # helps to construct the execute component item command as a post-executor
    # \brief It stores the new states of the current component
    def postExecute(self):
        if self._cp is not None:
            if self._cp.instance is None:
                self._cp.instance = Component(self.receiver.componentList)
                self._cp.instance.id = self._cp.id
                self._cp.instance.directory = \
                    self.receiver.componentList.directory
                self._cp.instance.name = \
                    self.receiver.componentList.elements[
                        self._cp.id].name

            if self._newstate is None and hasattr(
                    self._cp.instance, "getState"):
                self._newstate = self._cp.instance.getState()
            else:
                if hasattr(
                    self.receiver.componentList.elements[
                        self._cp.id].instance, "setState"):
                    self.receiver.componentList.elements[
                        self._cp.id].instance.setState(self._newstate)

                subwindow = self.receiver.subWindow(
                    self._cp.instance,
                    self.receiver.ui.mdi.subWindowList())

                if subwindow:
                    self.receiver.setActiveSubWindow(subwindow)
                    self._cp.instance.reconnectSaveAction()
                else:
                    self._cp.instance.createGUI()

                    self._cp.instance.addContextMenu(
                        self.receiver.contextMenuActions)
                    if self._cp.instance.isDirty():
                        self._cp.instance.dialog.setWindowTitle(
                            "%s [Component]*" % self._cp.name)
                    else:
                        self._cp.instance.dialog.setWindowTitle(
                            "%s [Component]" % self._cp.name)

                    self._cp.instance.reconnectSaveAction()
                    self._subwindow = self.receiver.ui.mdi.addSubWindow(
                        self._cp.instance.dialog)
                    self._subwindow.resize(680, 560)

                    if hasattr(self._cp.instance.dialog, "show"):
                        self._cp.instance.dialog.show()

                if hasattr(self._cp.instance, "connectExternalActions"):
                    self._cp.instance.connectExternalActions(
                        **self.receiver.externalCPActions)

        if hasattr(self._cp, "id"):
            self.receiver.componentList.populateElements(self._cp.id)
        else:
            self.receiver.componentList.populateElements()

    # executes the command
    # \brief It execute pre- and post- executors
    def redo(self):
        if self._cp is None:
            self.preExecute()

        self.postExecute()
        logger.debug("EXEC componentItemCommand")

    # helps to construct the unexecute component item command
    # \brief It changes back the states of the current component
    #        to the old state
    def undo(self):
        if self._cp is not None and self._oldstate is not None:
            self.receiver.componentList.elements[
                self._cp.id].instance.setState(self._oldstate)

            subwindow = self.receiver.subWindow(
                self._cp.instance, self.receiver.ui.mdi.subWindowList())
            if subwindow:
                self.receiver.setActiveSubWindow(subwindow)
            else:
                if self._cp.instance is None:
                    self._cp.instance = Component(self.receiver.componentList)
                    self._cp.instance.id = self._cp.id
                    self._cp.instance.directory = \
                        self.receiver.componentList.directory
                    self._cp.instance.name = \
                        self.receiver.componentList.elements[
                            self._cp.id].name
                if not self._cp.instance.dialog:
                    self._cp.instance.createGUI()
                self._subwindow = self.receiver.ui.mdi.addSubWindow(
                    self._cp.instance.dialog)
                self._subwindow.resize(680, 560)

                self._cp.instance.dialog.show()
        if self._cp.instance:
            self._cp.instance.reconnectSaveAction()
        if hasattr(self._cp, "id"):
            self.receiver.componentList.populateElements(self._cp.id)
        else:
            self.receiver.componentList.populateElements()

        logger.debug("UNDO componentItemComponent")


# Command which clears the whole current component
class ComponentClear(ComponentItemCommand):

    # constructor
    # \param receiver command receiver
    # \param parent command parent
    def __init__(self, receiver, parent=None):
        ComponentItemCommand.__init__(self, receiver, parent)

    # executes the command
    # \brief It clears the whole current component
    def redo(self):
        if self._cp is None:
            self.preExecute()
            if self._cp is not None:
                if QMessageBox.question(
                    self.receiver, "Component - Clear",
                    "Clear the component: %s ".encode() % (self._cp.name),
                        QMessageBox.Yes | QMessageBox.No,
                        QMessageBox.Yes) == QMessageBox.No:
                    self._oldstate = None
                    self._index = None
                    self._cp = None
                    return

                if hasattr(self._cp, "instance"):
                    if self._cp.instance in self.receiver.ui.mdi\
                            .subWindowList():
                        self.receiver.setActiveSubWindow(
                            self._cp.instance)
                    self._cp.instance.createHeader()

                    newModel = ComponentModel(
                        self._cp.instance.document,
                        self._cp.instance.getAttrFlag())
                    self._cp.instance.view.setModel(newModel)
                    self._cp.instance.connectView()

                    if hasattr(self._cp.instance,
                               "connectExternalActions"):
                        self._cp.instance.connectExternalActions(
                            **self.receiver.externalCPActions)

        self.postExecute()
        logger.debug("EXEC componentClear")


# Command which loads sub-components into the current component tree
#  from the file
class ComponentLoadComponentItem(ComponentItemCommand):

    # constructor
    # \param receiver command receiver
    # \param parent command parent
    def __init__(self, receiver, parent=None):
        ComponentItemCommand.__init__(self, receiver, parent)
        #
        self._itemName = ""

    # executes the command
    # \brief It loads sub-components into the current component
    #        tree from the file
    def redo(self):
        if self._cp is None:
            self.preExecute()
            if self._cp is not None:
                if self._cp.instance is not None and self._cp.instance.view \
                        and self._cp.instance.view.model():
                    if hasattr(self._cp.instance, "loadComponentItem"):
                        if not self._cp.instance.loadComponentItem(
                                self._itemName):
                            QMessageBox.warning(
                                self.receiver, "SubComponent not loaded",
                                "Please ensure that you have selected "
                                "the proper items")
                else:
                    QMessageBox.warning(
                        self.receiver, "Component item not selected",
                        "Please select one of the component items")

        self.postExecute()
        logger.debug("EXEC componentLoadcomponentItem")


# Command which moves the current component item into the clipboard
class ComponentRemoveItem(ComponentItemCommand):

    # constructor
    # \param receiver command receiver
    # \param parent command parent
    def __init__(self, receiver, parent=None):
        ComponentItemCommand.__init__(self, receiver, parent)

    # executes the command
    # \brief It moves the current component item into the clipboard
    def redo(self):
        if self._cp is None:
            self.preExecute()
            if self._cp is not None:
                if self._cp.instance is not None:
                    if hasattr(self._cp.instance, "removeSelectedItem"):
                        if not self._cp.instance.removeSelectedItem():
                            QMessageBox.warning(
                                self.receiver,
                                "Cutting item not possible",
                                "Please select another tree item")
        self.postExecute()
        logger.debug("EXEC componentRemoveItem")


# Command which copies the current component item into the clipboard
class ComponentCopyItem(ComponentItemCommand):

    # constructor
    # \param receiver command receiver
    # \param parent command parent
    def __init__(self, receiver, parent=None):
        ComponentItemCommand.__init__(self, receiver, parent)

    # executes the command
    # \brief It copies the current component item into the clipboard
    def redo(self):
        if self._cp is None:
            self.preExecute()
            if self._cp is not None:
                if self._cp.instance is not None:
                    if hasattr(self._cp.instance, "copySelectedItem"):
                        if not self._cp.instance.copySelectedItem():
                            QMessageBox.warning(
                                self.receiver,
                                "Copying item not possible",
                                "Please select another tree item")
        self.postExecute()
        logger.debug("EXEC componentCopyItem")


# Command which pastes the component item from the clipboard into
#  the current component tree
class ComponentPasteItem(ComponentItemCommand):

    # constructor
    # \param receiver command receiver
    # \param parent command parent
    def __init__(self, receiver, parent=None):
        ComponentItemCommand.__init__(self, receiver, parent)

    # executes the command
    # \brief It pastes the component item from the clipboard into
    #        the current component tree
    def redo(self):
        if self._cp is None:
            self.preExecute()
            if self._cp is not None:
                if self._cp.instance is not None:
                    if hasattr(self._cp.instance, "pasteItem"):
                        if not self._cp.instance.pasteItem():
                            QMessageBox.warning(
                                self.receiver,
                                "Pasting item not possible",
                                "Please select another tree item")
        self.postExecute()
        logger.debug("EXEC componentPasteItem")


# Command which moves the current, i.e. datasource or component item,
#  into the clipboard
class CutItem(ComponentItemCommand):

    # constructor
    # \param receiver command receiver
    # \param parent command parent
    def __init__(self, receiver, parent=None):
        ComponentItemCommand.__init__(self, receiver, parent)
        # type of the cutting item with values: component of datasource
        self.type = None

        self._ds = DataSourceCut(receiver, parent)
        self._cp = ComponentRemoveItem(receiver, parent)

    # executes the command
    # \brief It moves the current, i.e. datasource or component item,
    #        into the clipboard
    def redo(self):
        if self.type == 'component':
            self._cp.redo()
        elif self.type == 'datasource':
            self._ds.redo()

    # unexecutes the command
    # \brief It adds back the current, i.e. datasource or component item
    def undo(self):
        if self.type == 'component':
            self._cp.undo()
        elif self.type == 'datasource':
            self._ds.undo()


# Command which copies the current item, i.e. datasource or component item,
#  into the clipboard
class CopyItem(ComponentItemCommand):

    # constructor
    # \param receiver command receiver
    # \param parent command parent
    def __init__(self, receiver, parent=None):
        ComponentItemCommand.__init__(self, receiver, parent)
        # type of the coping item with values: component of datasource
        self.type = None

        self._ds = DataSourceCopy(receiver, parent)
        self._cp = ComponentCopyItem(receiver, parent)

    # executes the command
    # \brief It copies the current item, i.e. datasource or component item,
    #        into the clipboard
    def redo(self):
        if self.type == 'component':
            self._cp.redo()
        elif self.type == 'datasource':
            self._ds.redo()

    # unexecutes the command
    # \brief It unexecutes copy commands for datasources or components
    def undo(self):
        if self.type == 'component':
            self._cp.undo()
        elif self.type == 'datasource':
            self._ds.undo()


# Command which pastes the current item from the clipboard
#  into the current dialog, i.e. the current datasource or
#  the current component item tree
class PasteItem(QUndoCommand):

    # constructor
    # \param receiver command receiver
    # \param parent command parent
    def __init__(self, receiver, parent=None):
        QUndoCommand.__init__(self, parent)
        # main window
        self.receiver = receiver
        # element type
        self.type = None

        self._ds = DataSourcePaste(receiver, parent)
        self._cp = ComponentPasteItem(receiver, parent)

    # executes the command
    # \brief It pastes the current item from the clipboard into
    #        the current dialog, i.e. the current datasource or
    #        the current component item tree
    def redo(self):
        if self.type == 'component':
            self._cp.redo()
        elif self.type == 'datasource':
            self._ds.redo()

    # unexecutes the command
    # \brief It unexecutes paste commands for datasources or components
    def undo(self):
        if self.type == 'component':
            self._cp.undo()
        elif self.type == 'datasource':
            self._ds.undo()


# Command which merges the current component
class ComponentMerge(ComponentItemCommand):

    # constructor
    # \param receiver command receiver
    # \param parent command parent
    def __init__(self, receiver, parent=None):
        ComponentItemCommand.__init__(self, receiver, parent)

    # executes the command
    # \brief It merges the current component
    def redo(self):
        if self._cp is None:
            self.preExecute()
            if self._cp is not None:
                if hasattr(self._cp.instance, "merge"):
                    self._cp.instance.merge()

        self.postExecute()
        logger.debug("EXEC componentMerge")


# Command which creates a new item in the current component tree
class ComponentNewItem(ComponentItemCommand):

    # constructor
    # \param receiver command receiver
    # \param parent command parent
    def __init__(self, receiver, parent=None):
        ComponentItemCommand.__init__(self, receiver, parent)
        # name of the new component item
        self.itemName = ""
        self._index = None
        self._childIndex = None
        self._child = None

    # executes the command
    # \brief It creates a new item in the current component tree
    def redo(self):
        if self._cp is None:
            self.preExecute()
            if self._cp is not None:
                if self._cp.instance is None:
                    QMessageBox.warning(
                        self.receiver, "Component Item not selected",
                        "Please select one of the component Items")
                if hasattr(self._cp.instance, "addItem"):
                    self._child = self._cp.instance.addItem(self.itemName)
                    if self._child:
                        self._index = self._cp.instance.view.currentIndex()

                        if self._index.column() != 0 \
                                and self._index.row() is not None:

                            self._index = self._cp.instance.view.model().index(
                                self._index.row(), 0, self._index.parent())
                        row = self._cp.instance.dialog.getWidgetNodeRow(
                            self._child)
                        if row is not None:
                            self._childIndex = \
                                self._cp.instance.view.model().index(
                                    row, 0, self._index)
                            self._cp.instance.view.setCurrentIndex(
                                self._childIndex)
                            self._cp.instance.tagClicked(self._childIndex)
                    else:
                        QMessageBox.warning(
                            self.receiver,
                            "Creating the %s Item not possible"
                            % self.itemName,
                            "Please select another tree or new item ")
            if self._child and self._index.isValid():
                if self._index.isValid():
                    finalIndex = self._cp.instance.view.model().index(
                        self._index.parent().row(), 2,
                        self._index.parent().parent())
                else:
                    finalIndex = self._cp.instance.view.model().index(
                        0, 2, self._index.parent().parent())

                self._cp.instance.view.model().dataChanged.emit(
                    self._index, self._index)
                self._cp.instance.view.model().dataChanged.emit(
                    finalIndex, self._childIndex)

        self.postExecute()
        logger.debug("EXEC componentNewItem")


# Command which loads a datasource from a file into the current component tree
class ComponentLoadDataSourceItem(ComponentItemCommand):

    # constructor
    # \param receiver command receiver
    # \param parent command parent
    def __init__(self, receiver, parent=None):
        ComponentItemCommand.__init__(self, receiver, parent)
        self._cp = None
        # name of the new datasource item
        self.itemName = ""

    # executes the command
    # \brief It loads a datasource from a file into the current component tree
    def redo(self):
        if self._cp is None:
            self.preExecute()
            if self._cp is not None:
                if self._cp.instance is not None and self._cp.instance.view \
                        and self._cp.instance.view.model():
                    if hasattr(self._cp.instance, "loadDataSourceItem"):
                        if not self._cp.instance.loadDataSourceItem(
                                self.itemName):
                            QMessageBox.warning(
                                self.receiver, "DataSource not loaded",
                                "Please ensure that you have selected "
                                "the proper items")
                else:
                    QMessageBox.warning(
                        self.receiver, "Component item not selected",
                        "Please select one of the component items")

        self.postExecute()
        logger.debug("EXEC componentMerge")


# Command which adds the current datasource into the current component tree
class ComponentAddDataSourceItem(ComponentItemCommand):

    # constructor
    # \param receiver command receiver
    # \param parent command parent
    def __init__(self, receiver, parent=None):
        ComponentItemCommand.__init__(self, receiver, parent)

    # executes the command
    # \brief It adds the current datasource into the current component tree
    def redo(self):
        if self._cp is None:
            self.preExecute()
            if self._cp is not None:
                if self._cp.instance is None or \
                        self._cp.instance.view is None \
                        or self._cp.instance.view.model() is None:
                    self._oldstate = None
                    self._index = None
                    self._cp = None
                    QMessageBox.warning(
                        self.receiver, "Component Item not created",
                        "Please edit one of the component Items")
                    return

                ds = self.receiver.sourceList.currentListElement()
                if ds is None:
                    self._oldstate = None
                    self._index = None
                    self._cp = None
                    QMessageBox.warning(
                        self.receiver, "DataSource not selected",
                        "Please select one of the datasources")
                    return

                if ds.instance is None:
                    dsEdit = DataSource.DataSource(self.receiver.sourceList)
                    dsEdit.id = ds.id
                    dsEdit.directory = self.receiver.sourceList.directory
                    dsEdit.name = self.receiver.sourceList.elements[
                        ds.id].name
                    ds.instance = dsEdit
                else:
                    dsEdit = ds.instance

                if hasattr(dsEdit, "connectExternalActions"):
                    dsEdit.connectExternalActions(
                        **self.receiver.externalDSActions)

                if not hasattr(ds.instance, "createNodes"):
                    self._cp = None
                    QMessageBox.warning(
                        self.receiver, "Component Item not created",
                        "Please edit one of the component Items")
                    return

                dsNode = ds.instance.createNodes()
                if dsNode is None:
                    self._cp = None
                    QMessageBox.warning(
                        self.receiver,
                        "Datasource node cannot be created",
                        "Problem in importing the external node")
                    return

                if not hasattr(self._cp.instance, "addDataSourceItem"):
                    self._cp = None
                    QMessageBox.warning(
                        self.receiver, "Component Item not created",
                        "Please edit one of the component Items")
                    return
                if not self._cp.instance.addDataSourceItem(dsNode):
                    QMessageBox.warning(
                        self.receiver,
                        "Adding the datasource item not possible",
                        "Please ensure that you have selected "
                        "the proper items")

        self.postExecute()
        logger.debug("EXEC componentAddDataSourceItem")


# Command which links the current datasource into the current component tree
class ComponentLinkDataSourceItem(ComponentItemCommand):

    # constructor
    # \param receiver command receiver
    # \param parent command parent
    def __init__(self, receiver, parent=None):
        ComponentItemCommand.__init__(self, receiver, parent)

    # executes the command
    # \brief It links the current datasource into the current component tree
    def redo(self):
        if self._cp is None:
            self.preExecute()
            if self._cp is not None:
                if self._cp.instance is None \
                        or self._cp.instance.view is None \
                        or self._cp.instance.view.model() is None:
                    self._oldstate = None
                    self._index = None
                    self._cp = None
                    QMessageBox.warning(
                        self.receiver, "Component Item not created",
                        "Please edit one of the component Items")
                    return

                ds = self.receiver.sourceList.currentListElement()
                if ds is None:
                    self._oldstate = None
                    self._index = None
                    self._cp = None
                    QMessageBox.warning(
                        self.receiver, "DataSource not selected",
                        "Please select one of the datasources")
                    return

                if ds.instance is None:
                    dsEdit = DataSource.DataSource(self.receiver.sourceList)
                    dsEdit.id = ds.id
                    dsEdit.directory = self.receiver.sourceList.directory
                    dsEdit.name = self.receiver.sourceList.elements[
                        ds.id].name
                    ds.instance = dsEdit
                else:
                    dsEdit = ds.instance

                if not hasattr(ds.instance, "dataSourceName") \
                        or not ds.instance.dataSourceName:
                    self._cp = None
                    QMessageBox.warning(
                        self.receiver, "DataSource wihtout name",
                        "Please define datasource name")
                    return

                if hasattr(dsEdit, "connectExternalActions"):
                    dsEdit.connectExternalActions(
                        **self.receiver.externalDSActions)

                if not hasattr(ds.instance, "createNodes"):
                    self._cp = None
                    QMessageBox.warning(
                        self.receiver, "Component Item not created",
                        "Please edit one of the component Items")
                    return

                if not hasattr(self._cp.instance, "linkDataSourceItem"):
                    self._cp = None
                    QMessageBox.warning(
                        self.receiver, "Component Item not created",
                        "Please edit one of the component Items")
                    return
                if not self._cp.instance.linkDataSourceItem(
                        dsEdit.dataSourceName):
                    QMessageBox.warning(
                        self.receiver,
                        "Linking the datasource item not possible",
                        "Please ensure that you have selected "
                        "the proper items")

        self.postExecute()
        logger.debug("EXEC componentLinkDataSourceItem")


# Command which applies the changes from the form for
#  the current component item
class ComponentApplyItem(ComponentItemCommand):

    # constructor
    # \param receiver command receiver
    # \param parent command parent
    def __init__(self, receiver, parent=None):
        ComponentItemCommand.__init__(self, receiver, parent)

    # executes the command
    # \brief It applies the changes from the form for
    #        the current component item
    def redo(self):
        if self._cp is None:
            self.preExecute()
            if self._cp is not None:
                if hasattr(self._cp, "instance") \
                        and hasattr(self._cp.instance, "applyItem"):
                    if not self._cp.instance.applyItem():
                        pass
#                        QMessageBox.warning(
#                            self.receiver, "Applying item not possible",
#                            "Please select another tree item")

        self.postExecute()
        logger.debug("EXEC componentApplyItem")


# Command which move the current component item up
class ComponentMoveUpItem(ComponentItemCommand):

    # constructor
    # \param receiver command receiver
    # \param parent command parent
    def __init__(self, receiver, parent=None):
        ComponentItemCommand.__init__(self, receiver, parent)
        self._index = None

    # executes the command
    # \brief It applies the changes from the form for the current
    #    component item
    def redo(self):
        if self._cp is None:
            self.preExecute()
            if self._cp is not None:
                if hasattr(self._cp, "instance") and hasattr(
                        self._cp.instance, "moveUpItem"):

                    if self._cp.instance.moveUpItem() is None:
                        QMessageBox.warning(
                            self.receiver, "Moving item up not possible",
                            "Please select another tree item")

        self.postExecute()

        logger.debug("EXEC componentMoveUpItem")


# Command which move the current component item down
class ComponentMoveDownItem(ComponentItemCommand):

    # constructor
    # \param receiver command receiver
    # \param parent command parent
    def __init__(self, receiver, parent=None):
        ComponentItemCommand.__init__(self, receiver, parent)
        self._index = None

    # executes the command
    # \brief It applies the changes from the form for the current
    #    component item
    def redo(self):
        if self._cp is None:
            self.preExecute()
            if self._cp is not None:
                if hasattr(self._cp, "instance") \
                        and hasattr(self._cp.instance, "moveDownItem"):

                    if self._cp.instance.moveDownItem() is None:
                        QMessageBox.warning(
                            self.receiver,
                            "Moving item down not possible",
                            "Please select another tree item")

        self.postExecute()
        logger.debug("EXEC componentMoveDownItem")


if __name__ == "__main__":
    pass



#!/usr/bin/env python
#   This file is part of nexdatas - Tango Server for NeXus data writer
#
#    Copyright (C) 2012-2017 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
# \package nxsconfigtool nexdatas
# \file DimensionsDlg.py
# Dimensions dialog class

""" dimensions widget """

from PyQt5.QtCore import (Qt, )
from PyQt5.QtWidgets import (QTableWidgetItem, QMessageBox, QDialog)
from PyQt5 import uic

import os
import sys

import logging
# message logger
logger = logging.getLogger("nxsdesigner")

_formclass, _baseclass = uic.loadUiType(
    os.path.join(os.path.dirname(os.path.abspath(__file__)),
                 "ui", "dimensionsdlg.ui"))

if sys.version_info > (3,):
    unicode = str
    long = int


# dialog defining a dimensions tag
class DimensionsDlg(QDialog):

    # constructor
    # \param parent patent instance
    def __init__(self, parent=None):
        super(DimensionsDlg, self).__init__(parent)

        # dimensions rank
        self.rank = 0
        # dimensions lengths
        self.lengths = []

        # user interface
        self.ui = _formclass()

        # allowed subitems
        self.subItems = ["dim"]

    #  creates GUI
    # \brief It calls setupUi and  connects signals and slots
    def createGUI(self):

        try:
            if self.rank is not None and int(self.rank) >= 0:
                self.rank = int(self.rank)
                for i, ln in enumerate(self.lengths):
                    if ln:
                        if '$var.' not in ln and '$datasources.' not in ln:
                            iln = int(ln)
                            self.lengths[i] = ln
                            if iln < 1:
                                self.lengths[i] = None
                        else:
                            self.lengths[i] = ln
                    else:
                        self.lengths[i] = None
        except Exception:
            self.rank = 1
            self.lengths = []

        if not self.lengths:
            self.lengths = []

        self.ui.setupUi(self)

        self.ui.rankSpinBox.setValue(self.rank)

        self.ui.dimTableWidget.itemChanged.connect(
            self.__tableItemChanged)

        self.ui.dimTableWidget.setSortingEnabled(False)
        self.__populateLengths()
        self.ui.rankSpinBox.setFocus()

        self.ui.rankSpinBox.valueChanged[int].connect(self.__valueChanged)

    # takes a name of the current dim
    # \returns name of the current dim
    def __currentTableDim(self):
        return self.ui.dimTableWidget.currentRow()

    # changes the current value of the dim
    # \brief It changes the current value of the dim
    # and informs the user about wrong values
    def __tableItemChanged(self, item):
        row = self.__currentTableDim()

        if row not in range(len(self.lengths)):
            return
        column = self.ui.dimTableWidget.currentColumn()
        if column == 0:
            try:
                if item.text():
                    if '$var' not in str(item.text()) and  \
                            '$datasources' not in str(item.text()):
                        iln = int(item.text())
                        if iln < 1:
                            raise ValueError("Non-positive length value")
                    ln = str(item.text())
                    self.lengths[row] = ln
                else:
                    self.lengths[row] = None
            except Exception:
                QMessageBox.warning(
                    self, "Value Error", "Wrong value of the edited length")

        self.__populateLengths()

    # calls updateUi when the name text is changing
    # \param text the edited text
    def __valueChanged(self):
        self.rank = int(self.ui.rankSpinBox.value())
        self.__populateLengths(self.rank - 1)

    # fills in the dim table
    # \param selectedDim selected dim
    def __populateLengths(self, selectedDim=None):
        selected = None
        self.ui.dimTableWidget.clear()
        self.ui.dimTableWidget.setRowCount(self.rank)

        while self.rank > len(self.lengths):
            self.lengths.append(None)

        headers = ["Length"]
        self.ui.dimTableWidget.setColumnCount(len(headers))
        self.ui.dimTableWidget.setHorizontalHeaderLabels(headers)
        self.ui.dimTableWidget.setVerticalHeaderLabels(
            [unicode(l + 1) for l in range(self.rank)])
        for row, ln in enumerate(self.lengths):
            if ln:
                item = QTableWidgetItem(unicode(ln))
            else:
                item = QTableWidgetItem("")
            item.setData(Qt.UserRole, (long(row)))
            if selectedDim is not None and selectedDim == row:
                selected = item
            self.ui.dimTableWidget.setItem(row, 0, item)
        self.ui.dimTableWidget.resizeColumnsToContents()
        self.ui.dimTableWidget.horizontalHeader().setStretchLastSection(True)
        if selected is not None:
            selected.setSelected(True)
            self.ui.dimTableWidget.setCurrentItem(selected)
            self.ui.dimTableWidget.editItem(selected)

    # accepts input text strings
    # \brief It copies the dimensions name and type from lineEdit widgets
    #        and accept the dialog
    def accept(self):
        while len(self.lengths) > self.rank:
            self.lengths.pop()
        QDialog.accept(self)


if __name__ == "__main__":
    import sys
    from PyQt5.QtGui import QApplication

    logging.basicConfig(level=logging.DEBUG)

    # Qt application
    app = QApplication(sys.argv)
    # dimensions form
    form = DimensionsDlg()
    form.rank = 2
    form.lengths = [25, 27]
#    form.lengths = [None, 3]
    form.createGUI()
    form.show()
    app.exec_()

    if form.result():
        if form.rank:
            logger.info("Dimensions: rank = %s" % (form.rank))
        if form.lengths:
            logger.info("Lengths:")
            for mrow, mln in enumerate(form.lengths):
                logger.info(" %s: %s " % (mrow + 1, mln))



#!/usr/bin/env python
#   This file is part of nexdatas - Tango Server for NeXus data writer
#
#    Copyright (C) 2012-2017 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
# \package nxsconfigtool nexdatas
# \file ConfigurationServer.py
# Class for connecting to configuration server

""" Provides connects to configuration server"""

import logging
import time

from .ConnectDlg import ConnectDlg


try:
    import PyTango
    # if module PyTango avalable
    PYTANGO_AVAILABLE = True
    logger = logging.getLogger("nxsdesigner")
except ImportError as e:
    PYTANGO_AVAILABLE = False
    logger = logging.getLogger("nxsdesigner")
    logger.info("PyTango is not available: %s" % e)


# configuration server
class ConfigurationServer(object):

    # constructor
    def __init__(self):
        # name of tango device
        self.device = None

        # host name of tango device
        self.host = None

        # port of tango device
        self.port = None

        # connection status
        self.connected = False

        # device proxy
        self._proxy = None

    # sets server from string
    # \param device string
    def setServer(self, device):
        logger.debug(device)
        ahost = device.split(":")
        if len(ahost) > 1:
            self.host = ahost[0]
            adev = ahost[1].split("/")
            if len(adev) > 1:
                try:
                    self.port = int(adev[0])
                except Exception:
                    self.port = 10000
                self.device = "/".join(adev[1:])
            else:
                self.device = adev[0]
                self.port = 10000
        else:
            self.device = device
            self.host = 'localhost'
            self.port = 10000

    # allows to store the server state
    # \returns the state of the server
    def getState(self):
        return (self.device, self.host, self.port, self.connected)

    # allows to store the server state
    # \param state the state of the server
    def setState(self, state):
        (self.device, self.host, self.port, self.connected) = state

    def getDeviceName(self):
        if self.host and self.port:
            return "%s:%s/%s" % (str(self.host),
                                 str(self.port),
                                 str(self.device))
        else:
            return str(self.device)

    # connects to the configuration server
    # \brief It opens the configuration Tango device
    def connect(self):
        self._proxy = PyTango.DeviceProxy(self.getDeviceName())
        if self._proxy:
            found = False
            cnt = 0
            while not found and cnt < 1000:
                if cnt > 1:
                    time.sleep(0.01)
                try:
                    if self._proxy.state() != PyTango.DevState.RUNNING:
                        found = True
                except Exception:

                    time.sleep(0.01)
                    found = False
                cnt += 1

            if found:
                self._proxy.set_timeout_millis(25000)
                self._proxy.set_source(PyTango.DevSource.DEV)
                self._proxy.command_inout("Open")
                self.connected = True
            else:
                raise Exception("Cannot connect to: %s" % str(self.device))

    # opens connection to the configuration server
    # \brief It fetches parameters of tango device and calls connect() method
    def open(self):
        aform = ConnectDlg()
        if self.device:
            aform.device = self.device
        if self.host:
            aform.host = self.host
        if self.port is not None:
            aform.port = self.port

        aform.createGUI()
        aform.show()
        if aform.exec_():
            self.device = aform.device
            self.host = aform.host
            self.port = aform.port
            self.connect()

    # fetch all components
    # \returns dictionary with names : xml of components
    def fetchComponents(self):
        names = []
        comps = []
        if self._proxy and self.connected:
            names = self._proxy.command_inout("AvailableComponents")

            try:
                comps = self._proxy.command_inout("Components", names)
            except Exception:
                comps = []
                for n in names:
                    try:
                        xml = self._proxy.command_inout("Components", [n])
                        comps.append(xml[0])
                    except Exception:
                        comps.append("")
            return dict(zip(names, comps))

    # fetch all datasources
    # \returns dictionary with names : xml of datasources
    def fetchDataSources(self):
        names = []
        ds = []
        if self._proxy and self.connected:
            names = self._proxy.command_inout("AvailableDataSources")
            try:
                ds = self._proxy.command_inout("DataSources", names)
            except Exception:
                ds = []
                for n in names:
                    try:
                        xml = self._proxy.command_inout("DataSources", [n])
                        ds.extend(xml)
                    except Exception:
                        ds.append("")

            return dict(zip(names, ds))

    # stores the component
    # \param name component name
    # \param xml XML content of the component
    def storeComponent(self, name, xml):
        if self._proxy and self.connected:
            self._proxy.XMLString = str(xml)
            self._proxy.command_inout("StoreComponent", str(name))

    # stores the datasource
    # \param name datasource name
    # \param xml XML content of the datasource
    def storeDataSource(self, name, xml):
        if self._proxy and self.connected:
            self._proxy.XMLString = str(xml)
            self._proxy.command_inout("StoreDataSource", str(name))

    # stores the component
    # \param name component name
    def deleteComponent(self, name):
        if self._proxy and self.connected:
            self._proxy.command_inout("DeleteComponent", str(name))

    # stores the datasource
    # \param name datasource name
    def deleteDataSource(self, name):
        if self._proxy and self.connected:
            self._proxy.command_inout("DeleteDataSource", str(name))

    # set the given component mandatory
    # \param name component name
    def setMandatory(self, name):
        if self._proxy and self.connected:
            self._proxy.command_inout("SetMandatoryComponents",
                                      [str(name)])

    # get the mandatory components
    # returns list of the mandatory components
    def getMandatory(self):
        if self._proxy and self.connected:
            return self._proxy.command_inout("MandatoryComponents")

    # unset the given component mandatory
    # \param name component name
    def unsetMandatory(self, name):
        if self._proxy and self.connected:
            self._proxy.command_inout("UnsetMandatoryComponents", [str(name)])

    # closes connecion
    # \brief It closes connecion to configuration server
    def close(self):
        if self._proxy and self.connected:
            if self._proxy.state() == PyTango.DevState.OPEN:
                self._proxy.command_inout("Close")
                self.connected = False


# test function
def test():
    import sys
    from PyQt5.QtGui import QApplication

    app = QApplication(sys.argv)
    cs = ConfigurationServer()
    cs.device = "p09/mcs/r228"
    cs.host = "haso228k.desy.de"
    cs.port = 10000
    cs.open()
    cs.close()
    app.exit()


if __name__ == "__main__":
    test()



#!/usr/bin/env python
#   This file is part of nexdatas - Tango Server for NeXus data writer
#
#    Copyright (C) 2012-2017 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
# \package nxsconfigtool nexdatas
# \file LinkDlg.py
# Link dialog class

""" link widget """

import os
import sys

from PyQt5.QtCore import (QModelIndex)
from PyQt5.QtWidgets import (QMessageBox)
from PyQt5 import uic

from .NodeDlg import NodeDlg
from .Errors import CharacterError
from .DomTools import DomTools

import logging
# message logger
logger = logging.getLogger("nxsdesigner")

_formclass, _baseclass = uic.loadUiType(
    os.path.join(os.path.dirname(os.path.abspath(__file__)),
                 "ui", "linkdlg.ui"))

if sys.version_info > (3,):
    unicode = str


# dialog defining a tag link
class LinkDlg(NodeDlg):

    # constructor
    # \param parent patent instance
    def __init__(self, parent=None):
        super(LinkDlg, self).__init__(parent)

        # link name
        self.name = u''
        # link target
        self.target = u''
        # field doc
        self.doc = u''

        # allowed subitems
        self.subItems = ["doc", "datasource", "strategy"]

        # user interface
        self.ui = _formclass()

    # updates the link dialog
    # \brief It sets the form local variables
    def updateForm(self):
        if self.name is not None:
            self.ui.nameLineEdit.setText(self.name)
        if self.doc is not None:
            self.ui.docTextEdit.setText(self.doc)

        if self.target is not None:
            self.ui.targetLineEdit.setText(self.target)

    #  creates GUI
    # \brief It calls setupUi and  connects signals and slots
    def createGUI(self):

        self.ui.setupUi(self)

        self.updateForm()

        self._updateUi()

        self.ui.resetPushButton.clicked.connect(self.reset)
        self.ui.nameLineEdit.textEdited[str].connect(
            self._updateUi)

    # provides the state of the link dialog
    # \returns state of the group in tuple
    def getState(self):

        state = (self.name,
                 self.target,
                 self.doc
                 )
        return state

    # sets the state of the link dialog
    # \param state link state written in tuple
    def setState(self, state):

        (self.name,
         self.target,
         self.doc
         ) = state

    # sets the form from the DOM node
    # \param node DOM node
    def setFromNode(self, node=None):
        if node:
            # defined in NodeDlg
            self.node = node
        if not self.node:
            return
        attributeMap = self.node.attributes()

        self.name = unicode(
            attributeMap.namedItem("name").nodeValue()
            if attributeMap.contains("name") else "")
        self.target = unicode(
            attributeMap.namedItem("target").nodeValue()
            if attributeMap.contains("target") else "")

        text = unicode(DomTools.getText(self.node)).strip()
        if text and text.startswith("$datasources."):
            self.target = text

        doc = self.node.firstChildElement(str("doc"))
        text = DomTools.getText(doc)
        self.doc = unicode(text).strip() if text else ""

    # updates link user interface
    # \brief It sets enable or disable the OK button
    def _updateUi(self):
        enable = bool(self.ui.nameLineEdit.text())
        self.ui.applyPushButton.setEnabled(enable)

    # accepts input text strings
    # \brief It copies the link name and target from lineEdit widgets
    #        and accept the dialog
    def apply(self):
        name = unicode(self.ui.nameLineEdit.text())

        try:
            if 1 in [c in name for c in '!"#$%&\'()*+,/;<=>?@[\\]^`{|}~']:
                raise CharacterError(
                    "Name contains one of forbidden characters")
            if name[0] == '-':
                raise CharacterError(
                    "The first character of Name is '-'")

        except CharacterError as e:
            QMessageBox.warning(self, "Character Error", unicode(e))
            return
        self.name = name
        self.target = unicode(self.ui.targetLineEdit.text())

        self.doc = unicode(self.ui.docTextEdit.toPlainText())

        index = self.view.currentIndex()
        finalIndex = self.view.model().createIndex(
            index.row(), 2, index.parent().internalPointer())
        self.view.expand(index)

        if self.node and self.root and self.node.isElement():
            self.updateNode(index)

        if index.column() != 0:
            index = self.view.model().index(index.row(), 0, index.parent())
        self.view.model().dataChanged.emit(index, finalIndex)
        self.view.expand(index)

    # updates the Node
    # \brief It sets node from the dialog variables
    def updateNode(self, index=QModelIndex()):
        elem = self.node.toElement()
        mindex = self.view.currentIndex() if not index.isValid() else index

        attributeMap = self.node.attributes()
        for _ in range(attributeMap.count()):
            attributeMap.removeNamedItem(attributeMap.item(0).nodeName())
        if self.name:
            elem.setAttribute(str("name"), str(self.name))
        self.replaceText(mindex)
        if self.target:
            if self.target.startswith("$datasources."):
                self.replaceText(mindex, unicode(self.target))
            else:
                elem.setAttribute(str("target"), str(self.target))

        doc = self.node.firstChildElement(str("doc"))
        if not self.doc and doc and doc.nodeName() == "doc":
            self.removeElement(doc, mindex)
        elif self.doc:
            newDoc = self.root.createElement(str("doc"))
            newText = self.root.createTextNode(str(self.doc))
            newDoc.appendChild(newText)
            if doc and doc.nodeName() == "doc":
                self.replaceElement(doc, newDoc, mindex)
            else:
                self.appendElement(newDoc, mindex)


if __name__ == "__main__":
    import sys
    from PyQt5.QtGui import QApplication

    logging.basicConfig(level=logging.DEBUG)

    # Qt application
    app = QApplication(sys.argv)
    # link form
    form = LinkDlg()
    form.name = 'data'
    form.target = '/NXentry/NXinstrument/NXdetector/data'
    form.createGUI()
    form.show()
    app.exec_()

    if form.name:
        logger.info("Link: %s = \'%s\'" % (form.name, form.target))



#!/usr/bin/env python
#   This file is part of nexdatas - Tango Server for NeXus data writer
#
#    Copyright (C) 2012-2017 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
# \package nxsconfigtool nexdatas
# \file MainWindow.py
# Main window of the application

""" main window application dialog """

import os
import sys

from PyQt5.QtCore import (
    QSettings, Qt, pyqtSlot)
from PyQt5.QtGui import (
    QIcon,
    QTextCursor)
from PyQt5.QtWidgets import (
    QFrame, QUndoGroup, QUndoStack,
    QMainWindow, QMessageBox, QLabel)
from PyQt5 import uic

# from .ui.ui_mainwindow import Ui_MainWindow

from .DataSourceList import DataSourceList
from .ComponentList import ComponentList
from .DataSourceDlg import CommonDataSourceDlg
from .ComponentDlg import ComponentDlg

from .FileSlots import FileSlots
from .ListSlots import ListSlots
from .EditSlots import EditSlots
from .ItemSlots import ItemSlots
from .ServerSlots import ServerSlots
from .HelpSlots import HelpSlots
from .WindowsSlots import WindowsSlots
from .Logger import LogStream, LogActions

from .ConfigurationServer import (ConfigurationServer, PYTANGO_AVAILABLE)
from .ComponentCreator import (NXSTOOLS_AVAILABLE)

import logging

# message logger
logger = logging.getLogger("nxsdesigner")

_formclass, _baseclass = uic.loadUiType(
    os.path.join(os.path.dirname(os.path.abspath(__file__)),
                 "ui", "mainwindow.ui"))

if sys.version_info > (3,):
    unicode = str


def iternext(it):
    if sys.version_info > (3,):
        return next(iter(it.values()))
    else:
        return it.itervalues().next()


# main window class
class MainWindow(QMainWindow):

    # constructor
    # \param components component direcotry
    # \param datasources datasource directory
    # \param server configuration server
    # \param parent parent widget
    def __init__(self, components=None, datasources=None,
                 server=None, parent=None):
        super(MainWindow, self).__init__(parent)
        logger.debug("PARAMETERS: %s %s %s %s",
                     components, datasources, server, parent)

        # component tree menu under mouse cursor
        self.contextMenuActions = None

        # slots for DataSource widget buttons
        self.externalDSActions = {}
        # datasource list menu under mouse cursor
        self.dsourceListMenuActions = None
        # list of datasources
        self.sourceList = None
        # datasource directory label
        self.dsDirLabel = None

        # slots for Component widget buttons
        self.externalCPActions = {}
        # component list menu under mouse cursor
        self.componentListMenuActions = None
        # list of components
        self.componentList = None
        # component directory label
        self.cpDirLabel = None

        # stack with used commands
        self.undoStack = None
        # group of command stacks
        self.undoGroup = None

        # user interface
        self.ui = _formclass()

        # configuration server
        self.configServer = None

        # online.xml file name
        self.onlineFile = None

        # action slots
        self.slots = {}

        # log actions
        self.logActions = LogActions(self)

        settings = QSettings()
        dsDirectory = self.__setDirectory(
            settings, "DataSources/directory", "datasources",
            datasources)
        cpDirectory = self.__setDirectory(
            settings, "Components/directory", "components",
            components)

        self.createGUI(dsDirectory, cpDirectory)
        self.createActions()

        if self.componentList:
            self.componentList.setActions(self.componentListMenuActions)
        if self.sourceList:
            self.sourceList.setActions(self.dsourceListMenuActions)

        self.loadDataSources()
        self.loadComponents()

        mgeo = settings.value("MainWindow/Geometry")
        if mgeo is not None:
            self.restoreGeometry(mgeo)
        mst = settings.value("MainWindow/State")
        if mst is not None:
            self.restoreState(mst)

        if PYTANGO_AVAILABLE:
            self.setupServer(settings, server)

        status = self.createStatusBar()
        status.showMessage("Ready", 5000)
        self.setWindowTitle("NXS Component Designer")

    def dataSourceContent(self, did):
        if logger.getEffectiveLevel() > 20:
            return
        ds = self.sourceList.elements[did]
        components = []
        datasources = []
        message = ""
        if ds and ds.name:
            if ds and ds.name and ds.instance:
                name = ds.instance.name
                components = self.componentList.dataSourceComponents(name)
                datasources = ds.instance.datasources
            message = "datasource '%s' " % name
            if name != ds.name:
                message += "('%s')" % (ds.name)
            if components:
                message += "\n    is linked to: '%s' components" % \
                           "', '".join([
                               str(el) for el in components])
            if datasources:
                message += "\n    depends on: '%s' datasources" % \
                           "', '".join([
                               str(el) for el in datasources])
        if message:
            logger.info(message)

    def componentContent(self, did):
        if logger.getEffectiveLevel() > 20:
            return
        cp = self.componentList.elements[did]
        components = []
        datasources = []
        message = ""
        if cp and cp.name:
            if cp and cp.name and cp.instance:
                name = cp.instance.name
                components = cp.instance.components
                datasources = cp.instance.datasources
            message = "component '%s' " % name
            if name != cp.name:
                message += "('%s')" % (cp.name)
            if components:
                message += "\n    depends on: '%s' components" % \
                           "', '".join([
                               str(el) for el in components])
            if datasources:
                message += "\n    depends on: '%s' datasources" % \
                           "', '".join([
                               str(el) for el in datasources])
        if message:
            logger.info(message)

    #  creates GUI
    # \brief It create dialogs for the main window application
    # \param dsDirectory datasource directory
    # \param cpDirectory component directory
    def createGUI(self, dsDirectory, cpDirectory):
        self.ui.setupUi(self)

        self.sourceList = DataSourceList(dsDirectory, self)
        self.sourceList.createGUI()

        self.componentList = ComponentList(cpDirectory, self)
        self.componentList.createGUI()

        self.ui.dockSplitter.addWidget(self.componentList)
        self.ui.dockSplitter.addWidget(self.sourceList)
        self.ui.dockSplitter.setStretchFactor(0, 2)
        self.ui.dockSplitter.setStretchFactor(1, 1)
        self.connectLogger()

    @pyqtSlot(bool)
    def debug(self, flag):
        self.logActions.setlevel("debug", flag)

    @pyqtSlot(bool)
    def info(self, flag):
        self.logActions.setlevel("info", flag)

    @pyqtSlot(bool)
    def warning(self, flag):
        self.logActions.setlevel("warning", flag)

    @pyqtSlot(bool)
    def error(self, flag):
        self.logActions.setlevel("error", flag)

    @pyqtSlot(bool)
    def critical(self, flag):
        self.logActions.setlevel("critical", flag)

    @pyqtSlot(str)
    def insertText(self, message):
        self.ui.logTextBrowser.insertPlainText(message)
        self.ui.logTextBrowser.moveCursor(QTextCursor.End)

    def connectLogger(self):
        self.ui.actionDebug.triggered.connect(self.debug)
        self.ui.actionInfo.triggered.connect(self.info)
        self.ui.actionWarning.triggered.connect(self.warning)
        self.ui.actionError.triggered.connect(self.error)
        self.ui.actionCritical.triggered.connect(self.critical)
        LogStream.stdout().written.connect(self.insertText)
        self.logActions.updatelevel()
        doc = self.ui.logTextBrowser.document()
        doc.setMaximumBlockCount(1000)

    # setups direcconfiguration server
    # \param settings application QSettings object
    # \param name setting variable name
    # \param default defualt value
    # \param directory user's directory
    # \returns set directory
    @classmethod
    def __setDirectory(cls, settings, name, default, directory=None):
        if directory and os.path.exists(directory):
            return os.path.abspath(directory)
        ldir = ""
        dsdir = unicode(settings.value(name))
        if dsdir:
            ldir = os.path.abspath(dsdir)
        else:
            if os.path.exists(os.path.join(os.getcwd(), default)):
                ldir = os.path.abspath(
                    os.path.join(os.getcwd(), default))
            else:
                ldir = os.getcwd()
        return ldir

    # setups configuration server
    # \param settings application QSettings object
    # \param server user's server
    def setupServer(self, settings, server=None):

        self.configServer = ConfigurationServer()
        if server:
            self.configServer.setServer(server)
        else:
            self.configServer.device = unicode(
                settings.value("ConfigServer/device"))
            self.configServer.host = unicode(
                settings.value("ConfigServer/host"))
            self.onlineFile = unicode(
                settings.value("Online/filename"))
            port = str(settings.value("ConfigServer/port") or "")
            if port:
                self.configServer.port = int(port)

    # updates directories in status bar
    def updateStatusBar(self):
        self.cpDirLabel.setText("CP: %s" % (self.componentList.directory))
        self.dsDirLabel.setText("DS: %s" % (self.sourceList.directory))

    # creates status bar
    # \returns status bar
    def createStatusBar(self):
        status = self.statusBar()
        status.setSizeGripEnabled(False)
        self.cpDirLabel = QLabel("CP: %s" % (self.componentList.directory))
        self.cpDirLabel.setFrameStyle(QFrame.Panel | QFrame.Sunken)
        self.dsDirLabel = QLabel("DS: %s" % (self.sourceList.directory))
        self.dsDirLabel.setFrameStyle(QFrame.Panel | QFrame.Sunken)
        status.addWidget(QLabel(""), 4)
        status.addWidget(self.cpDirLabel, 4)
        status.addWidget(self.dsDirLabel, 4)
        return status

    # creates action
    # \param action the action instance
    # \param text string shown in menu
    # \param slot action slot
    # \param shortcut key short-cut
    # \param icon qrc_resource icon name
    # \param tip text for status bar and text hint
    # \param checkable if command/action checkable
    # \param signal action signal
    def __setAction(self, action, _, slot=None, shortcut=None, icon=None,
                    tip=None, checkable=False, signal="triggered"):
        if icon is not None:
            action.setIcon(QIcon(":/%s.png" % unicode(icon).strip()))
        if shortcut is not None:
            action.setShortcut(shortcut)
        if tip is not None:
            action.setToolTip(tip)
            action.setStatusTip(tip)
        if slot is not None:
            # self.connect(action, SIGNAL(signal), slot)
            getattr(action, signal).connect(slot)
        if checkable:
            action.setCheckable(True)
        return action

    def __setActions(self, slots):
        for ac, pars in slots.actions.items():
            action = getattr(self.ui, ac)
            self.__setAction(
                action, pars[0],
                getattr(slots, pars[1]),
                pars[2], pars[3], pars[4])

    def __setTasks(self, slots):
        if hasattr(slots, "tasks"):
            for pars in slots.tasks:
                # self.connect(pars[1], SIGNAL(pars[2]),
                #             getattr(slots, pars[0]))
                getattr(pars[1], pars[2]).connect(getattr(slots, pars[0]))

    def __createUndoRedoActions(self):
        self.undoGroup.addStack(self.undoStack)
        self.undoGroup.setActiveStack(self.undoStack)
        actionUndo = self.undoGroup.createUndoAction(self)
        actionUndo.setIcon(QIcon(":/undo.png"))
        actionRedo = self.undoGroup.createRedoAction(self)
        actionRedo.setIcon(QIcon(":/redo.png"))

        actionUndo.setToolTip("Undo")
        actionUndo.setStatusTip("Undo")
        actionRedo.setToolTip("Redo")
        actionRedo.setStatusTip("Redo")

        actionUndo.setShortcut("Ctrl+Z")
        actionRedo.setShortcut("Ctrl+Y")

        self.ui.menuEdit.insertAction(self.ui.menuEdit.actions()[0],
                                      actionUndo)
        self.ui.menuEdit.insertAction(actionUndo, actionRedo)
        self.ui.editToolBar.addAction(actionUndo)
        self.ui.editToolBar.addAction(actionRedo)

    # creates actions
    # \brief It creates actions and sets the command pool and stack
    def createActions(self):
        self.undoGroup = QUndoGroup(self)
        self.undoStack = QUndoStack(self)

        self.__createUndoRedoActions()

        self.slots["File"] = FileSlots(self)
        self.slots["List"] = ListSlots(self)
        self.slots["Edit"] = EditSlots(self)
        self.slots["Item"] = ItemSlots(self)
        self.slots["Server"] = ServerSlots(self)
        self.slots["Help"] = HelpSlots(self)
        self.slots["Windows"] = WindowsSlots(self)

        for sl in self.slots.values():
            self.__setActions(sl)
            self.__setTasks(sl)

        self.slots["Windows"].updateWindowMenu()

        if not PYTANGO_AVAILABLE:
            self.ui.actionConnectServer.setDisabled(True)
        self.disableServer(True)

        viewCompDockAction = self.ui.compDockWidget.toggleViewAction()
        viewCompDockAction.setToolTip("Show/Hide the dock lists")
        viewCompDockAction.setStatusTip("Show/Hide the dock lists")

        viewLogDockAction = self.ui.logDockWidget.toggleViewAction()
        viewLogDockAction.setToolTip("Show/Hide the logger")
        viewLogDockAction.setStatusTip("Show/Hide the logger")

        self.ui.menuView.insertAction(
            self.ui.menuView.actions()[0],
            viewLogDockAction)

        self.ui.menuView.insertAction(
            self.ui.menuView.actions()[0],
            viewCompDockAction)

        self.__setAction(
            self.ui.actionAllAttributesView,
            "&All Attributes", self.viewAllAttributes, "",
            tip="Go to the component list", checkable=True)

        # Signals
        # self.connect(self.componentList.ui.elementListWidget,
        #              SIGNAL("itemDoubleClicked(QListWidgetItem*)"),
        #              self.slots["Edit"].componentEdit)
        self.componentList.ui.elementListWidget.itemDoubleClicked.connect(
            self.slots["Edit"].componentEdit)

        # self.connect(self.sourceList.ui.elementListWidget,
        #              SIGNAL("itemDoubleClicked(QListWidgetItem*)"),
        #              self.slots["Edit"].dsourceEdit)
        self.sourceList.ui.elementListWidget.itemDoubleClicked.connect(
            self.slots["Edit"].dsourceEdit)

        # Component context menu
        self.ui.mdi.setContextMenuPolicy(Qt.ActionsContextMenu)
        self.contextMenuActions = (
            self.ui.actionNewGroupItem,
            self.ui.actionNewFieldItem,
            self.ui.actionNewDataSourceItem,
            self.ui.actionNewStrategyItem,
            self.ui.actionNewAttributeItem,
            self.ui.actionNewLinkItem,
            None,
            self.ui.actionLoadSubComponentItem,
            self.ui.actionLoadDataSourceItem,
            None,
            self.ui.actionAddDataSourceItem,
            self.ui.actionLinkDataSourceItem,
            None,
            self.ui.actionCutItem,
            self.ui.actionCopyItem,
            self.ui.actionPasteItem,
            self.ui.actionTakeDataSourceItem,
            None,
            self.ui.actionMoveUpComponentItem,
            self.ui.actionMoveDownComponentItem,
            None,
            self.ui.actionApplyComponentItem,
            None,
            self.ui.actionClearComponentItems
        )

        # Component list menu
        self.componentListMenuActions = (
            self.ui.actionNew,
            self.ui.actionEditComponent,
            self.ui.actionMergeComponentItems,
            self.ui.actionClose,
            None, {"File": (
                self.ui.actionLoad,
                self.ui.actionSave,
                self.ui.actionSaveAs,
                self.ui.actionSaveAll,
                self.ui.actionReloadList,
                self.ui.actionChangeDirectory)},
            None, {"Server": (
                self.ui.actionFetchComponentsServer,
                self.ui.actionStoreComponentServer,
                self.ui.actionStoreAllComponentsServer,
                self.ui.actionDeleteComponentServer,
                self.ui.actionGetMandatoryComponentsServer,
                self.ui.actionSetComponentMandatoryServer,
                self.ui.actionUnsetComponentMandatoryServer)},
            None,
            self.ui.actionTakeDataSources
        )

        # DataSource list menu
        self.dsourceListMenuActions = (
            self.ui.actionNewDataSource,
            self.ui.actionEditDataSource,
            self.ui.actionApplyDataSource,
            self.ui.actionCloseDataSource,
            None,
            self.ui.actionCopyDataSource,
            self.ui.actionCutDataSource,
            self.ui.actionPasteDataSource,
            None, {"File": (
                self.ui.actionLoadDataSource,
                self.ui.actionSaveDataSource,
                self.ui.actionSaveDataSourceAs,
                self.ui.actionSaveAllDataSources,
                self.ui.actionReloadDataSourceList,
                self.ui.actionChangeDataSourceDirectory)},
            None, {"Server": (
                self.ui.actionFetchDataSourcesServer,
                self.ui.actionStoreDataSourceServer,
                self.ui.actionStoreAllDataSourcesServer,
                self.ui.actionDeleteDataSourceServer)}
        )

        # datasource widget actions
        self.externalDSActions = {
            "externalSave": self.slots["File"].dsourceSaveButton,
            "externalApply": self.slots["Edit"].dsourceApplyButton,
            "externalClose": self.slots["Windows"].dsourceClose,
            "externalStore": self.slots[
                "Server"].serverStoreDataSourceButton}

        # component widget actions
        self.externalCPActions = {
            "externalSave": self.slots["File"].componentSaveButton,
            "externalStore": self.slots["Server"].serverStoreComponentButton,
            "externalApply": self.slots["Item"].componentApplyItemButton,
            "externalClose": self.slots["Windows"].componentClose,
            "externalDSLink": self.slots[
                "Item"].componentLinkDataSourceItemButton}

    # stores the list element before finishing the application
    # \param event Qt event
    # \param elementList element list
    # \param failures a list of errors
    # \returns True if not canceled
    def closeList(self, event, elementList, failures):
        status = None
        for k in elementList.elements.keys():
            cp = elementList.elements[k]
            if (hasattr(cp, "isDirty") and cp.isDirty()) or \
                    (hasattr(cp, "instance")
                     and hasattr(cp.instance, "isDirty")
                     and cp.instance.isDirty()):
                if status != QMessageBox.YesToAll \
                        and status != QMessageBox.NoToAll:
                    status = QMessageBox.question(
                        self, "%s - Save" % elementList.clName,
                        "Do you want to save %s: %s"
                        % (elementList.clName, cp.name),
                        QMessageBox.Yes | QMessageBox.No | QMessageBox.Cancel
                        | QMessageBox.YesToAll | QMessageBox.NoToAll,
                        QMessageBox.Yes)

                if status == QMessageBox.Yes or status == QMessageBox.YesToAll:
                    try:
                        cid = elementList.currentListElement()
                        elementList.populateElements(cp.id)
                        if hasattr(cp.instance, "merge"):
                            self.slots["Edit"].componentEdit()
                            cp.instance.merge()
                        else:
                            self.slots["Edit"].dsourceEdit()
                        if not cp.instance.save():
                            elementList.populateElements(cid)
                            if event:
                                event.ignore()
                            return
                        elementList.populateElements(cid)

                    except IOError as e:
                        failures.append(unicode(e))

                elif status == QMessageBox.Cancel:
                    if event:
                        event.ignore()
                    return
        return True

    # Provides a name of the currently selected datasource
    def currentDataSourceName(self):
        name = ""
        if hasattr(self.sourceList.currentListElement(), "id"):
            dsid = self.sourceList.currentListElement().id
            ds = self.sourceList.elements[dsid]
            if ds and ds.name:
                name = ds.name
                if ds and ds.name and ds.instance:
                    name = ds.instance.name
        return name

    # Stores settings in QSettings object
    def __storeSettings(self):
        settings = QSettings()
        settings.setValue(
            "MainWindow/Geometry",
            (self.saveGeometry()))
        settings.setValue(
            "MainWindow/State",
            (self.saveState()))
        settings.setValue(
            "DataSources/directory",
            (os.path.abspath(self.sourceList.directory)))
        settings.setValue(
            "Components/directory",
            (os.path.abspath(self.componentList.directory)))

        if self.configServer:
            settings.setValue("ConfigServer/device",
                              (self.configServer.device))
            settings.setValue("ConfigServer/host",
                              (self.configServer.host))
            settings.setValue("ConfigServer/port",
                              (self.configServer.port))
            settings.setValue("ConfigServer/port",
                              (self.configServer.port))
            settings.setValue("Online/filename",
                              (self.onlineFile))
            self.configServer.close()

    # stores the setting before finishing the application
    # \param event Qt event
    def closeEvent(self, event):
        failures = []
        if not self.closeList(event, self.componentList, failures):
            return
        if not self.closeList(event, self.sourceList, failures):
            return
        if (failures and
            QMessageBox.warning(
                self, "NXS Component Designer -- Save Error",
                "Failed to save%s\nQuit anyway?"
                % unicode("\n\t".join(failures)),
                QMessageBox.Yes | QMessageBox.No) == QMessageBox.No):
            event.ignore()
            return

        self.__storeSettings()
        self.ui.mdi.closeAllSubWindows()

    # disables/enable the server actions
    # \param status True for disable
    def disableServer(self, status):
        self.ui.actionFetchComponentsServer.setDisabled(status)
        self.ui.actionStoreComponentServer.setDisabled(status)
        self.ui.actionStoreAllComponentsServer.setDisabled(status)
        self.ui.actionDeleteComponentServer.setDisabled(status)
        self.ui.actionGetMandatoryComponentsServer.setDisabled(status)
        self.ui.actionSetComponentMandatoryServer.setDisabled(status)
        self.ui.actionUnsetComponentMandatoryServer.setDisabled(status)
        self.ui.actionFetchDataSourcesServer.setDisabled(status)
        self.ui.actionStoreDataSourceServer.setDisabled(status)
        self.ui.actionStoreAllDataSourcesServer.setDisabled(status)
        self.ui.actionDeleteDataSourceServer.setDisabled(status)
        self.ui.actionCloseServer.setDisabled(status)
        self.ui.actionCreateComponentServer.setDisabled(
            status or not NXSTOOLS_AVAILABLE)
        self.ui.actionCreateStdComponentServer.setDisabled(
            status or not NXSTOOLS_AVAILABLE)
        self.ui.actionCreateDataSourcesServer.setDisabled(
            status or not NXSTOOLS_AVAILABLE)

        if self.configServer and self.configServer.device:
            dev = "%s:%s/%s" % (
                self.configServer.host
                if self.configServer.host else "localhost",
                str(self.configServer.port)
                if self.configServer.port else "10000",
                self.configServer.device
            )
        else:
            dev = "None"

        if status:
            self.setWindowTitle("NXS Component Designer -||- [%s]" % dev)
        else:
            self.setWindowTitle("NXS Component Designer <-> [%s]" % dev)

    # loads the datasource list
    # \brief It loads the datasource list from the default directory
    def loadDataSources(self):
        self.sourceList.loadList(self.externalDSActions)
        ide = iternext(self.sourceList.elements).id \
            if len(self.sourceList.elements) else None

        self.sourceList.populateElements(ide)

    # sets the datasource list from dictionary
    # \param datasources dictionary with datasources, i.e. name:xml
    # \param new logical variable set to True if objects are not saved
    def setDataSources(self, datasources, new=False):
        last = self.sourceList.setList(
            datasources,
            self.externalDSActions,
            None,
            new)
        ide = iternext(self.sourceList.elements).id \
            if len(self.sourceList.elements) else None

        self.sourceList.populateElements(ide)
        return last

    # sets the component list from the given dictionary
    # \param components dictionary with components, i.e. name:xml
    def setComponents(self, components):
        self.componentList.setList(
            components,
            self.externalCPActions,
            self.contextMenuActions
        )
        ide = iternext(self.componentList.elements).id \
            if len(self.componentList.elements) else None

        self.componentList.populateElements(ide)

    # loads the component list
    # \brief It loads the component list from the default directory
    def loadComponents(self):
        self.componentList.loadList(
            self.externalCPActions,
            self.contextMenuActions
        )
        ide = iternext(self.componentList.elements).id \
            if len(self.componentList.elements) else None

        self.componentList.populateElements(ide)

    # update datasource list item according to open window
    # \returns True if windows is open
    def updateDataSourceListItem(self):
        status = False
        if self.ui.mdi.activeSubWindow() and isinstance(
                self.ui.mdi.activeSubWindow().widget(),
                CommonDataSourceDlg):
            widget = self.ui.mdi.activeSubWindow().widget()
            if isinstance(widget, CommonDataSourceDlg):
                if widget.datasource.id is not None:
                    if hasattr(self.sourceList.currentListElement(), "id"):
                        if self.sourceList.currentListElement().id \
                                != widget.datasource.id:
                            self.sourceList.populateElements(
                                widget.datasource.id)
                    status = True
        return status

    # update component list item according to open window
    # \returns True if windows is open
    def updateComponentListItem(self):
        status = False
        if self.ui.mdi.activeSubWindow() and isinstance(
                self.ui.mdi.activeSubWindow().widget(), ComponentDlg):
            widget = self.ui.mdi.activeSubWindow().widget()
            if isinstance(widget, ComponentDlg):
                if widget.component.id is not None:

                    if hasattr(self.componentList.currentListElement(), "id"):
                        if self.componentList.currentListElement().id \
                                != widget.component.id:
                            self.componentList.populateElements(
                                widget.component.id)
                    status = True
        return status

    # actives sub-window if in mdi area
    def setActiveSubWindow(self, window):
        w = window if window in self.ui.mdi.subWindowList() else None
        self.ui.mdi.setActiveSubWindow(w)

    # deselect component list item according to open window
    def deselectComponentSubWindow(self):
        if self.ui.mdi.activeSubWindow() and isinstance(
                self.ui.mdi.activeSubWindow().widget(), ComponentDlg):
            widget = self.ui.mdi.activeSubWindow().widget()
            if isinstance(widget, ComponentDlg):
                if widget.component.id is not None:

                    if hasattr(self.componentList.currentListElement(), "id"):
                        if self.componentList.currentListElement().id \
                                != widget.component.id:

                            self.setActiveSubWindow(None)

    # deselect component list item according to open window
    def deselectDataSourceSubWindow(self):
        if self.ui.mdi.activeSubWindow() and isinstance(
                self.ui.mdi.activeSubWindow().widget(), CommonDataSourceDlg):
            widget = self.ui.mdi.activeSubWindow().widget()
            if isinstance(widget, CommonDataSourceDlg):
                if widget.datasource.id is not None:

                    if hasattr(self.sourceList.currentListElement(), "id"):
                        if self.sourceList.currentListElement().id \
                                != widget.datasource.id:

                            self.setActiveSubWindow(None)

    # shows all attributes in the tree
    # \brief switch between all attributes in the tree or only type attribute
    def viewAllAttributes(self):
        self.componentList.viewAttributes(
            not self.componentList.viewAttributes())

    # provides subwindow defined by instance
    # \param instance given instance
    # \param subwindows list of subwindows
    # \returns required subwindow
    @classmethod
    def subWindow(cls, instance, subwindows):
        swin = None
        for sw in subwindows:
            if hasattr(sw, "widget"):
                if hasattr(sw.widget(), "component") \
                        and sw.widget().component == instance:
                    swin = sw
                    break
                elif hasattr(sw.widget(), "datasource") \
                        and sw.widget().datasource == instance:
                    swin = sw
                    break
        return swin



#!/usr/bin/env python
#   This file is part of nexdatas - Tango Server for NeXus data writer
#
#    Copyright (C) 2012-2017 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
# \package nxsconfigtool nexdatas
# \file DefinitionDlg.py
# Definition dialog class

""" definition widget """

import copy
import os
import sys

from PyQt5.QtCore import (Qt, QModelIndex)
from PyQt5.QtWidgets import (QMessageBox, QTableWidgetItem)
from PyQt5 import uic

# from .ui.ui_definitiondlg import Ui_DefinitionDlg
from .AttributeDlg import AttributeDlg
from .NodeDlg import NodeDlg
from .DomTools import DomTools

import logging
# message logger
logger = logging.getLogger("nxsdesigner")


_formclass, _baseclass = uic.loadUiType(
    os.path.join(os.path.dirname(os.path.abspath(__file__)),
                 "ui", "definitiondlg.ui"))

if sys.version_info > (3,):
    unicode = str


# dialog defining a definition tag
class DefinitionDlg(NodeDlg):

    # constructor
    # \param parent patent instance
    def __init__(self, parent=None):
        super(DefinitionDlg, self).__init__(parent)

        # definition name
        self.name = u''
        # definition content
        self.content = u''
        # definition doc
        self.doc = u''
        # definition attributes
        self.attributes = {}
        self.__attributes = {}

        # allowed subitems
        self.subItems = ["group", "field", "attribute", "link", "component",
                         "doc", "symbols"]

        # user interface
        self.ui = _formclass()

    # updates the definition dialog
    # \brief It sets the form local variables
    def updateForm(self):

        if self.name is not None:
            self.ui.nameLineEdit.setText(self.name)
        if self.content is not None:
            self.ui.contentTextEdit.setText(self.content)
        if self.doc is not None:
            self.ui.docTextEdit.setText(self.doc)

        self.__attributes.clear()
        for at in self.attributes.keys():
            self.__attributes[unicode(at)] = self.attributes[(unicode(at))]

        self.populateAttributes()

    # provides the state of the definition dialog
    # \returns state of the definition in tuple
    def getState(self):
        attributes = copy.copy(self.attributes)

        state = (self.name,
                 self.content,
                 self.doc,
                 attributes
                 )
        return state

    # sets the state of the definition dialog
    # \param state definition state written in tuple
    def setState(self, state):

        (self.name,
         self.content,
         self.doc,
         attributes
         ) = state
        self.attributes = copy.copy(attributes)

    #  creates GUI
    # \brief It calls ui.setupUi(self),  updateForm() and
    #        connects signals and slots
    def createGUI(self):
        self.ui.setupUi(self)

        self.updateForm()

        self.ui.resetPushButton.clicked.connect(self.reset)
        self.ui.attributeTableWidget.itemChanged.connect(
            self.__tableItemChanged)
        self.ui.addPushButton.clicked.connect(self.__addAttribute)
        self.ui.removePushButton.clicked.connect(self.__removeAttribute)

    # sets the form from the DOM node
    # \param node DOM node
    def setFromNode(self, node=None):
        if node:
            # defined in NodeDlg class
            self.node = node
        if not self.node:
            # exception?
            return
        attributeMap = self.node.attributes()

        self.name = unicode(
            attributeMap.namedItem("name").nodeValue()
            if attributeMap.contains("name") else "")

        self.attributes.clear()
        self.__attributes.clear()
        for i in range(attributeMap.count()):
            attribute = attributeMap.item(i)
            attrName = unicode(attribute.nodeName())
            if attrName != "name":
                self.attributes[attrName] = unicode(attribute.nodeValue())
                self.__attributes[attrName] = unicode(attribute.nodeValue())

        text = DomTools.getText(self.node)
        self.content = unicode(text).strip() if text else ""

        doc = self.node.firstChildElement(str("doc"))
        text = DomTools.getText(doc)
        self.doc = unicode(text).strip() if text else ""

    # adds an attribute
    #  \brief It runs the Definition Dialog and fetches attribute name and
    #     value
    def __addAttribute(self):
        aform = AttributeDlg()
        if aform.exec_():
            if aform.name not in self.__attributes.keys():
                self.__attributes[aform.name] = aform.value
                self.populateAttributes(aform.name)
            else:
                QMessageBox.warning(
                    self, "Attribute name exists",
                    "To change the attribute value, please edit the value "
                    "in the attribute table")

    # takes a name of the current attribute
    # \returns name of the current attribute
    def __currentTableAttribute(self):
        item = self.ui.attributeTableWidget.item(
            self.ui.attributeTableWidget.currentRow(), 0)
        if item is None:
            return None
        return item.data(Qt.UserRole)

    # removes an attribute
    #  \brief It removes the current attribute asking before about it
    def __removeAttribute(self):
        attr = self.__currentTableAttribute()
        if attr is None:
            return
        if unicode(attr) in self.__attributes.keys():
            self.__attributes.pop(unicode(attr))
            self.populateAttributes()

    # changes the current value of the attribute
    # \brief It changes the current value of the attribute and informs
    #        the user that attribute names arenot editable
    def __tableItemChanged(self, item):
        attr = self.__currentTableAttribute()
        if unicode(attr) not in self.__attributes.keys():
            return
        column = self.ui.attributeTableWidget.currentColumn()
        if column == 1:
            self.__attributes[unicode(attr)] = unicode(item.text())
        if column == 0:
            QMessageBox.warning(
                self, "Attribute name is not editable",
                "To change the attribute name, please remove "
                "the attribute and add the new one")
        self.populateAttributes()

    # fills in the attribute table
    # \param selectedAttribute selected attribute
    def populateAttributes(self, selectedAttribute=None):
        selected = None
        self.ui.attributeTableWidget.clear()
        self.ui.attributeTableWidget.setSortingEnabled(False)
        self.ui.attributeTableWidget.setRowCount(len(self.__attributes))
        headers = ["Name", "Value"]
        self.ui.attributeTableWidget.setColumnCount(len(headers))
        self.ui.attributeTableWidget.setHorizontalHeaderLabels(headers)
        for row, name in enumerate(self.__attributes):
            item = QTableWidgetItem(name)
            item.setData(Qt.UserRole, (name))
            self.ui.attributeTableWidget.setItem(row, 0, item)
            item2 = QTableWidgetItem(self.__attributes[name])
            self.ui.attributeTableWidget.setItem(row, 1, item2)
            if selectedAttribute is not None and selectedAttribute == name:
                selected = item2
        self.ui.attributeTableWidget.setSortingEnabled(True)
        self.ui.attributeTableWidget.resizeColumnsToContents()
        self.ui.attributeTableWidget.horizontalHeader().\
            setStretchLastSection(True)
        if selected is not None:
            selected.setSelected(True)
            self.ui.attributeTableWidget.setCurrentItem(selected)

    # applys input text strings
    # \brief It copies the definition name and type from lineEdit widgets
    #        and apply the dialog
    def apply(self):
        self.name = unicode(self.ui.nameLineEdit.text())

        self.content = unicode(self.ui.contentTextEdit.toPlainText())
        self.doc = unicode(self.ui.docTextEdit.toPlainText())

        index = self.view.currentIndex()
        finalIndex = self.view.model().createIndex(
            index.row(), 2, index.parent().internalPointer())

        self.attributes.clear()
        for at in self.__attributes.keys():
            self.attributes[at] = self.__attributes[at]

        self.view.expand(index)
        if self.node and self.root and self.node.isElement():
            self.updateNode(index)

        if index.column() != 0:
            index = self.view.model().index(index.row(), 0, index.parent())
        self.view.model().dataChanged.emit(index, finalIndex)
        self.view.expand(index)

    # updates the Node
    # \param index current node index
    # \brief It sets node from the dialog variables
    def updateNode(self, index=QModelIndex()):
        elem = self.node.toElement()
        mindex = self.view.currentIndex() if not index.isValid() else index
        attributeMap = self.node.attributes()
        for _ in range(attributeMap.count()):
            attributeMap.removeNamedItem(attributeMap.item(0).nodeName())
        if self.name:
            elem.setAttribute(str("name"), str(self.name))

        self.replaceText(mindex, unicode(self.content))

        for attr in self.attributes.keys():
            elem.setAttribute(str(attr), str(self.attributes[attr]))

        doc = self.node.firstChildElement(str("doc"))
        if not self.doc and doc and doc.nodeName() == "doc":
            self.removeElement(doc, mindex)
        elif self.doc:
            newDoc = self.root.createElement(str("doc"))
            newText = self.root.createTextNode(str(self.doc))
            newDoc.appendChild(newText)
            if doc and doc.nodeName() == "doc":
                self.replaceElement(doc, newDoc, mindex)
            else:
                self.appendElement(newDoc, mindex)


if __name__ == "__main__":
    import sys
    from PyQt5.QtGui import QApplication

    logging.basicConfig(level=logging.DEBUG)

    # Qt application
    app = QApplication(sys.argv)
    # definition form
    form = DefinitionDlg()
    form.name = 'scan'
    form.content = '$components.default'
    form.doc = 'The main entry'
    form.attributes = {"title": "Test run 1", "run_cycle": "2012-1"}
    form.createGUI()
    form.show()
    app.exec_()

    if form.nexusType:
        logger.info("Definition: name = \'%s\' type = \'%s\'"
                    % (form.name, form.nexusType))
    if form.attributes:
        logger.info("Other attributes:")
        for k in form.attributes.keys():
            logger.info(" %s = '%s' " % (k, form.attributes[k]))
    if form.doc:
        logger.info("Doc: \n%s" % form.doc)



#!/usr/bin/env python
#   This file is part of nexdatas - Tango Server for NeXus data writer
#
#    Copyright (C) 2012-2017 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
# \package nxsconfigtool nexdatas
# \file RichAttributeDlg.py
# Rich Attribute dialog class

""" attribute widget """

import copy
import os

from PyQt5.QtCore import (QModelIndex)
from PyQt5.QtWidgets import QMessageBox
from PyQt5 import uic

from .NodeDlg import NodeDlg
from .DimensionsDlg import DimensionsDlg
from .Errors import CharacterError
from .DomTools import DomTools

import logging
import sys
# message logger
logger = logging.getLogger("nxsdesigner")

_formclass, _baseclass = uic.loadUiType(
    os.path.join(os.path.dirname(os.path.abspath(__file__)),
                 "ui", "richattributedlg.ui"))

if sys.version_info > (3,):
    unicode = str


# dialog defining an attribute
class RichAttributeDlg(NodeDlg):

    # constructor
    # \param parent patent instance
    def __init__(self, parent=None):
        super(RichAttributeDlg, self).__init__(parent)

        # attribute name
        self.name = u''
        # attribute value
        self.value = u''
        # attribute type
        self.nexusType = u''
        # attribute doc
        self.doc = u''

        # rank
        self.rank = 0
        # dimensions
        self.dimensions = []
        self._dimensions = []

        # allowed subitems
        self.subItems = ["enumeration", "doc", "datasource",
                         "strategy", "dimensions"]

        # user interface
        self.ui = _formclass()

    # provides the state of the richattribute dialog
    # \returns state of the richattribute in tuple
    def getState(self):
        dimensions = copy.copy(self.dimensions)

        state = (self.name,
                 self.value,
                 self.nexusType,
                 self.doc,
                 self.rank,
                 dimensions
                 )
        return state

    # sets the state of the richattribute dialog
    # \param state richattribute state written in tuple
    def setState(self, state):

        (self.name,
         self.value,
         self.nexusType,
         self.doc,
         self.rank,
         dimensions
         ) = state
        self.dimensions = copy.copy(dimensions)

    # links dataSource
    # \param dsName datasource name
    def linkDataSource(self, dsName):
        self.value = "$%s.%s" % (self.dsLabel, dsName)
        self.updateForm()

    # updates the richattribute dialog
    # \brief It sets the form local variables
    def updateForm(self):
        if self.name is not None:
            self.ui.nameLineEdit.setText(self.name)
        if self.nexusType is not None:
            index = self.ui.typeComboBox.findText(unicode(self.nexusType))
            if index > -1:
                self.ui.typeComboBox.setCurrentIndex(index)
                self.ui.otherFrame.hide()
            else:
                index2 = self.ui.typeComboBox.findText('other ...')
                self.ui.typeComboBox.setCurrentIndex(index2)
                self.ui.typeLineEdit.setText(self.nexusType)
                self.ui.otherFrame.show()
        else:
            index = self.ui.typeComboBox.findText(unicode("None"))
            self.ui.typeComboBox.setCurrentIndex(index)
            self.ui.otherFrame.hide()

        if self.doc is not None:
            self.ui.docTextEdit.setText(self.doc)
        if self.value is not None:
            self.ui.valueLineEdit.setText(self.value)

        if self.rank < len(self.dimensions):
            self.rank = len(self.dimensions)

        if self.dimensions:
            label = self.dimensions.__str__()
            self.ui.dimLabel.setText("%s" % label.replace('None', '*'))
        elif self.rank > 0:
            label = ([None] * self.rank).__str__()
            self.ui.dimLabel.setText("%s" % label.replace('None', '*'))
        else:
            self.ui.dimLabel.setText("[]")

        self._dimensions = []
        for dm in self.dimensions:
            self._dimensions.append(dm)

    #  creates GUI
    # \brief It calls setupUi and  connects signals and slots
    def createGUI(self):
        self.ui.setupUi(self)

        self.updateForm()

        self._updateUi()

        self.ui.resetPushButton.clicked.connect(self.reset)

        self.ui.nameLineEdit.textEdited[str].connect(self._updateUi)
        self.ui.typeComboBox.currentIndexChanged[str].connect(
            self._currentIndexChanged)
        self.ui.dimPushButton.clicked.connect(self._changeDimensions)

    # sets the form from the DOM node
    # \param node DOM node
    def setFromNode(self, node=None):
        if node:
            # defined in NodeDlg
            self.node = node
        if not self.node:
            return
        attributeMap = self.node.attributes()

        self.name = unicode(attributeMap.namedItem("name").nodeValue()
                            if attributeMap.contains("name") else "")
        self.nexusType = unicode(attributeMap.namedItem("type").nodeValue()
                                 if attributeMap.contains("type") else "")

        text = DomTools.getText(self.node)
        self.value = unicode(text).strip() if text else ""

        dimens = self.node.firstChildElement(str("dimensions"))
        attributeMap = dimens.attributes()

        self.dimensions = []
        self._dimensions = []
        if attributeMap.contains("rank"):
            try:
                self.rank = int(attributeMap.namedItem("rank").nodeValue())
                if self.rank < 0:
                    self.rank = 0
            except Exception:
                self.rank = 0
        else:
            self.rank = 0
        if self.rank > 0:
            child = dimens.firstChild()
            while not child.isNull():
                if child.isElement() and child.nodeName() == "dim":
                    attributeMap = child.attributes()
                    index = None
                    value = None
                    try:
                        if attributeMap.contains("index"):
                            index = int(attributeMap.namedItem(
                                "index").nodeValue())
                        if attributeMap.contains("value"):
                            value = str(attributeMap.namedItem(
                                "value").nodeValue())
                    except Exception:
                        pass
                    text = DomTools.getText(child)
                    if text and "$datasources." in text:
                        value = str(text).strip()
                    if index < 1:
                        index = None
                    if index is not None:
                        while len(self.dimensions) < index:
                            self.dimensions.append(None)
                            self._dimensions.append(None)
                        self._dimensions[index - 1] = value
                        self.dimensions[index - 1] = value

                child = child.nextSibling()

        if self.rank < len(self.dimensions):
            self.rank = len(self.dimensions)
            self.rank = len(self._dimensions)
        elif self.rank > len(self.dimensions):
            self.dimensions.extend(
                [None] * (self.rank - len(self.dimensions)))
            self._dimensions.extend(
                [None] * (self.rank - len(self._dimensions)))

        doc = self.node.firstChildElement(str("doc"))
        text = DomTools.getText(doc)
        self.doc = unicode(text).strip() if text else ""

    # changing dimensions of the field
    #  \brief It runs the Dimensions Dialog and fetches rank
    #         and dimensions from it
    def _changeDimensions(self):
        dform = DimensionsDlg(self)
        dform.rank = self.rank
        dform.lengths = [ln for ln in self._dimensions]
        dform.createGUI()
        if dform.exec_():
            self.rank = dform.rank
            if self.rank:
                self._dimensions = [ln for ln in dform.lengths]
            else:
                self._dimensions = []
            label = self._dimensions.__str__()
            self.ui.dimLabel.setText("%s" % label.replace('None', '*'))

    # calls updateUi when the name text is changing
    # \param text the edited text
    def _currentIndexChanged(self, text):
        if text == 'other ...':
            self.ui.otherFrame.show()
            self.ui.typeLineEdit.setFocus()
        else:
            self.ui.otherFrame.hide()

    # updates attribute user interface
    # \brief It sets enable or disable the OK button
    def _updateUi(self):
        enable = bool(self.ui.nameLineEdit.text())
        self.ui.applyPushButton.setEnabled(enable)

    # accepts input text strings
    # \brief It copies the attribute name and value from lineEdit widgets
    #        and accept the dialog
    def apply(self):
        name = unicode(self.ui.nameLineEdit.text())

        try:
            if 1 in [c in name for c in '!"#$%&\'()*+,/;<=>?@[\\]^`{|}~']:
                raise CharacterError(
                    "Name contains one of forbidden characters")
            if name[0] == '-':
                raise CharacterError(
                    "The first character of Name is '-'")
        except CharacterError as e:
            QMessageBox.warning(self, "Character Error", unicode(e))
            return

        self.name = name
        self.value = unicode(self.ui.valueLineEdit.text())

        self.nexusType = unicode(self.ui.typeComboBox.currentText())
        if self.nexusType == 'other ...':
            self.nexusType = unicode(self.ui.typeLineEdit.text())
        elif self.nexusType == 'None':
            self.nexusType = u''

        self.doc = unicode(self.ui.docTextEdit.toPlainText())

        index = self.view.currentIndex()
        finalIndex = self.view.model().createIndex(
            index.row(), 2, index.parent().internalPointer())
        self.view.expand(index)

        self.dimensions = []
        for dm in self._dimensions:
            self.dimensions.append(dm)

        if self.node and self.root and self.node.isElement():
            self.updateNode(index)
        if index.column() != 0:
            index = self.view.model().index(index.row(), 0, index.parent())
        self.view.model().dataChanged.emit(index, finalIndex)
        self.view.expand(index)

    # updates the Node
    # \brief It sets node from the dialog variables
    def updateNode(self, index=QModelIndex()):
        elem = self.node.toElement()
        mindex = self.view.currentIndex() if not index.isValid() else index

        attributeMap = self.node.attributes()
        for i in range(attributeMap.count()):
            attributeMap.removeNamedItem(attributeMap.item(0).nodeName())
        elem.setAttribute(str("name"), str(self.name))
        if self.nexusType:
            elem.setAttribute(str("type"), str(self.nexusType))

        self.replaceText(mindex, unicode(self.value))

        doc = self.node.firstChildElement(str("doc"))
        if not self.doc and doc and doc.nodeName() == "doc":
            self.removeElement(doc, mindex)
        elif self.doc:
            newDoc = self.root.createElement(str("doc"))
            newText = self.root.createTextNode(str(self.doc))
            newDoc.appendChild(newText)
            if doc and doc.nodeName() == "doc":
                self.replaceElement(doc, newDoc, mindex)
            else:
                self.appendElement(newDoc, mindex)

        dimens = self.node.firstChildElement(str("dimensions"))
        if not self.dimensions and dimens \
                and dimens.nodeName() == "dimensions":
            self.removeElement(dimens, mindex)
        elif self.dimensions:
            newDimens = self.root.createElement(str("dimensions"))
            newDimens.setAttribute(str("rank"),
                                   str(unicode(self.rank)))
            dimDefined = True
            for i in range(min(self.rank, len(self.dimensions))):
                if self.dimensions[i] is None:
                    dimDefined = False
            if dimDefined:
                for i in range(min(self.rank, len(self.dimensions))):
                    dim = self.root.createElement(str("dim"))
                    dim.setAttribute(str("index"), str(unicode(i + 1)))
                    if "$datasources." not in unicode(self.dimensions[i]):

                        dim.setAttribute(str("value"),
                                         str(unicode(self.dimensions[i])))
                    else:
                        dsText = self.root.createTextNode(
                            str(unicode(self.dimensions[i])))
                        dstrategy = self.root.createElement(
                            str("strategy"))
                        dstrategy.setAttribute(str("mode"),
                                               str(unicode("CONFIG")))
                        dim.appendChild(dsText)
                        dim.appendChild(dstrategy)
                    newDimens.appendChild(dim)

            if dimens and dimens.nodeName() == "dimensions":
                self.replaceElement(dimens, newDimens, mindex)
            else:
                self.appendElement(newDimens, mindex)

    # appends newElement
    # \param newElement DOM node to append
    # \param parent parent DOM node
    def appendElement(self, newElement, parent):
        singles = {"datasource": "DataSource", "strategy": "Strategy"}
        if unicode(newElement.nodeName()) in singles:
            if not self.node:
                return
            child = self.node.firstChild()
            while not child.isNull():
                if child.nodeName() == unicode(newElement.nodeName()):
                    QMessageBox.warning(
                        self, "%s exists" % singles[
                            str(newElement.nodeName())],
                        "To add a new %s please remove the old one"
                        % newElement.nodeName())
                    return False
                child = child.nextSibling()

        return NodeDlg.appendElement(self, newElement, parent)


if __name__ == "__main__":
    import sys
    from PyQt5.QtGui import QApplication

    logging.basicConfig(level=logging.DEBUG)

    # Qt application
    app = QApplication(sys.argv)
    # attribute form
    form = RichAttributeDlg()
    form.name = "pre_sample_flightpath"
    form.nexusType = 'NX_FLOAT'
    form.doc = "This is the flightpath before the sample position."
    form.value = "1.2"
    form.createGUI()
    form.show()
    app.exec_()

    if form.name:
        logger.info("Attribute: %s = \'%s\'" % (form.name, form.value))
    if form.nexusType:
        logger.info("Type: %s" % form.nexusType)
    if form.doc:
        logger.info("Doc: \n%s" % form.doc)



#!/usr/bin/env python
#   This file is part of nexdatas - Tango Server for NeXus data writer
#
#    Copyright (C) 2012-2017 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
# \package nxsconfigtool nexdatas
# \file HelpSlots.py
# user pool commands of GUI application

""" Help slots """

import platform
import sys

from PyQt5.QtGui import (QKeySequence)
from PyQt5.QtWidgets import (QMessageBox)
from PyQt5.QtCore import (QT_VERSION_STR, PYQT_VERSION_STR)

from . import __version__
from .HelpForm import HelpForm

if sys.version_info > (3,):
    unicode = str


# stack with the application commands
class HelpSlots(object):

    # constructor
    # \param main the main window dialog
    def __init__(self, main):
        # main window
        self.main = main
        # command stack
        self.undoStack = main.undoStack

        # action data
        self.actions = {
            "actionAboutComponentDesigner": [
                "&About Component Designer",
                "helpAbout", "", "icon", "About Component Designer"],
            "actionComponentDesignerHelp": [
                "Component Desigener &Help", "helpHelp",
                QKeySequence.HelpContents, "help", "Detail help"]
        }

    # shows help about
    # \brief It shows message box with help about
    def helpAbout(self):
        QMessageBox.about(
            self.main, "About Component Designer",
            """<b>Component Designer</b> v %s
            <p>Copyright &copy; 2012-2017 DESY, GNU GENERAL PUBLIC LICENSE
            <p>This application can be used to create
            XML configuration files for Nexus Data Writer.
            <p>Python %s - Qt %s - PyQt %s on %s""" % (
                unicode(__version__),
                unicode(platform.python_version()),
                unicode(QT_VERSION_STR),
                unicode(PYQT_VERSION_STR),
                unicode(platform.system())))

    # shows the detail help
    # \brief It shows the detail help from help directory
    def helpHelp(self):
        form = HelpForm("index.html", self.main)
        form.show()


if __name__ == "__main__":
    pass



#!/usr/bin/env python
#   This file is part of nexdatas - Tango Server for NeXus data writer
#
#    Copyright (C) 2012-2017 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
# \package nxsconfigtool nexdatas
# \file ComponentModel.py
# component classes

""" component model for tree view """

from PyQt5.QtCore import (QAbstractItemModel, Qt, QModelIndex)
from PyQt5.QtXml import QDomNode

from . ComponentItem import ComponentItem


# model for component tree
class ComponentModel(QAbstractItemModel):
    # constuctor
    # \param document DOM document
    # \param parent widget
    # \param allAttributes True if show all attributes in the tree
    def __init__(self, document, allAttributes, parent=None):
        super(ComponentModel, self).__init__(parent)

        # show all attribures or only the type attribute
        self.__allAttributes = allAttributes

        # root item of the tree
        self.__rootItem = ComponentItem(document)
        # index of the root item
        self.rootIndex = self.createIndex(0, 0, self.__rootItem)

    # provides access to the header data
    # \param section integer index of the table column
    # \param orientation orientation of the header
    # \param role access type of the header data
    # \returns header data defined for the given index and formated according
    #          to the role
    def headerData(self, section, orientation, role=Qt.DisplayRole):
        if orientation == Qt.Horizontal and role == Qt.DisplayRole:
            if section == 0:
                return "Name"
            elif section == 1:
                if self.__allAttributes:
                    return "Attributes"
                else:
                    return "Type"
            elif section == 2:
                return "Value"
            else:
                return None

    # switches between all attributes in the try or only type attribute
    # \param allAttributes all attributes are shown if True
    def setAttributeView(self, allAttributes):
        self.__allAttributes = allAttributes

    # provides read access to the model data
    # \param index of the model item
    # \param role access type of the data
    # \returns data defined for the given index and formated according
    #          to the role
    def data(self, index, role=Qt.DisplayRole):
        if not index.isValid():
            return None
        if role != Qt.DisplayRole:
            return None

        item = index.internalPointer()
        node = item.node

        attributeMap = node.attributes()
#        if node.nodeName() == 'xml':
#            return

        if index.column() == 0:
            name = None
            if attributeMap.contains("name"):
                name = attributeMap.namedItem("name").nodeValue()

            if name is not None:
                return str(node.nodeName() + ": " + name)
            else:
                return str(node.nodeName())
        elif index.column() == 1:
            if self.__allAttributes:
                attributes = []
                for i in range(attributeMap.count()):
                    attribute = attributeMap.item(i)
                    attributes.append(attribute.nodeName() + "=\""
                                      + attribute.nodeValue() + "\"")
                return str(" ".join(attributes) + "  ")
            else:
                return str(
                    (attributeMap.namedItem("type").nodeValue() + "  ")
                    if attributeMap.contains("type") else str("  "))

        elif index.column() == 2:
            return str(" ".join(node.nodeValue().split("\n")))
        else:
            return ""

    # provides flag of the model item
    # \param index of the model item
    # \returns flag defined for the given index and formated according
    #          to the role
    def flags(self, index):
        if not index.isValid():
            return Qt.ItemIsEnabled
        return Qt.ItemFlags(QAbstractItemModel.flags(self, index) |
                            Qt.ItemIsEnabled | Qt.ItemIsSelectable)

    # provides access to the item index
    # \param row integer index counting DOM child item
    # \param column integer index counting table column
    # \param parent index of the parent item
    # \returns index for the required model item
    def index(self, row, column, parent=QModelIndex()):
        if not self.hasIndex(row, column, parent):
            return QModelIndex()

        if not parent.isValid():
            parentItem = self.__rootItem
        else:
            parentItem = parent.internalPointer()

        childItem = parentItem.child(row)
        if childItem:
            return self.createIndex(row, column, childItem)
        else:
            return QModelIndex()

    # provides access to the parent index
    # \param child  child index
    # \returns parent index for the given child
    def parent(self, child):
        if not child.isValid():
            return QModelIndex()

        childItem = child.internalPointer()

        if not hasattr(childItem, "parent"):
            return QModelIndex()

        parentItem = childItem.parent

        if parentItem is None or parentItem == self.__rootItem:
            return QModelIndex()

        if parentItem is None:
            return QModelIndex()

#        if parentItem == self.__rootItem:
#            self.rootIndex

        return self.createIndex(parentItem.childNumber(), 0, parentItem)

    # provides number of the model rows
    # \param parent parent index
    # \returns number of the children for the given parent
    def rowCount(self, parent=QModelIndex()):
        if parent.column() > 0:
            return 0

        if not parent.isValid():
            parentItem = self.__rootItem
        else:
            parentItem = parent.internalPointer()

        if not hasattr(parentItem, "node") or parentItem.node is None:
            return 0
        return parentItem.node.childNodes().count()

    # provides number of the model columns
    # \param parent parent index
    # \returns 3 which corresponds to component tag tree, tag attributes,
    #          tag values
    def columnCount(self, parent=QModelIndex()):
        return 3

    # inserts the given rows into the model
    # \param position row integer index where rows should be inserted
    # \param node DOM node to append
    # \param parent index of the parent item
    # \returns True if parent exists
    def insertItem(self, position, node, parent=QModelIndex()):
        item = parent.internalPointer()
        if not item:
            return False

        self.beginInsertRows(parent, position, position)

        pIndex = self.index(position, 0, parent)
        previous = pIndex.internalPointer().node if pIndex.isValid() \
            else QDomNode()
        item.node.insertBefore(node, previous)
#
        status = item.insertChildren(position, 1)

        self.endInsertRows()

        return status

    # append the given DOM node into parent item
    # \param node DOM node to append
    # \param parent index of the parent item
    # \returns True if parent exists
    def appendItem(self, node, parent=QModelIndex()):
        item = parent.internalPointer()
        if not item:
            return False
        position = item.node.childNodes().count()
        self.beginInsertRows(parent, position, position)

        pIndex = self.index(position, 0, parent)
        previous = pIndex.internalPointer().node \
            if pIndex.isValid() else QDomNode()
        item.node.insertAfter(node, previous)
#
        status = item.insertChildren(position, 1)

        self.endInsertRows()

        return status

    # removes the given rows from the model
    # \param position row integer index of the first removed row
    # \param parent index of the parent item
    # \returns True if parent exists
    def removeItem(self, position, parent=QModelIndex()):
        item = parent.internalPointer()
        if not item:
            return False
        self.beginRemoveRows(parent, position, position)
        node = item.child(position).node

        status = item.removeChildren(position, 1)
        item.node.removeChild(node)
        self.endRemoveRows()
        return status


if __name__ == "__main__":
    pass



#!/usr/bin/env python
#   This file is part of nexdatas - Tango Server for NeXus data writer
#
#    Copyright (C) 2012-2017 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
# \package nxsconfigtool nexdatas
# \file DataSourceMethods.py
# Data Source method class

""" Provides datasource widget methods """

import sys

from PyQt5.QtCore import (QModelIndex)
from PyQt5.QtWidgets import (QApplication, QMessageBox, QWidget, QVBoxLayout)
from PyQt5.QtXml import (QDomDocument)

from .DomTools import DomTools
from .Errors import ParameterError


import logging
# message logger
logger = logging.getLogger("nxsdesigner")

if sys.version_info > (3,):
    unicode = str


# dialog defining datasource
class DataSourceMethods(object):

    # constructor
    # \param dialog datasource dialog
    # \param datasource data
    # \param parent qt parent
    def __init__(self, dialog, datasource, parent=None):

        # datasource dialog
        self.__dialog = dialog

        # datasource data
        self.__datasource = datasource

        # qt parent
        self.__parent = parent

    # clears the dialog
    # \brief It sets dialog to None
    def setDialog(self, dialog=None):
        self.__dialog = dialog

    # creates a new dialog
    def createDialog(self):
        if self.__datasource.dialog:
            self.__dialog = self.__datasource.dialog
        else:
            self.__datasource.createDialog()

    # rejects the changes
    # \brief It asks for the cancellation  and reject the changes
    def close(self):
        if QMessageBox.question(self.__parent, "Close datasource",
                                "Would you like to close the datasource?",
                                QMessageBox.Yes | QMessageBox.No,
                                QMessageBox.Yes) == QMessageBox.No:
            return
        self.updateForm()
        if not self.__dialog:
            self.createDialog()
        self.__dialog.reject()

    #  resets the form
    # \brief It reverts form variables to the last accepted ones
    def reset(self):
        self.updateForm()

    # updates the datasource self.__dialog
    # \brief It sets the form local variables
    def updateForm(self):
        if not self.__dialog or not self.__datasource:
            raise ParameterError("updateForm parameters not defined")

        if self.__datasource.doc is not None:
            self.__dialog.ui.docTextEdit.setText(self.__datasource.doc)
        if self.__datasource.dataSourceType is not None:
            index = self.__dialog.ui.typeComboBox.findText(
                unicode(self.__datasource.dataSourceType))
            if index > -1:
                self.__dialog.ui.typeComboBox.setCurrentIndex(index)
            else:
                self.__datasource.dataSourceType = 'CLIENT'
        if self.__datasource.dataSourceName is not None:
            self.__dialog.ui.nameLineEdit.setText(
                self.__datasource.dataSourceName)

        for k in self.__dialog.imp.keys():
            self.__dialog.imp[k].updateForm(self.__datasource)

        self.__dialog.setFrames(self.__datasource.dataSourceType)

    # sets the tree mode used in ComponentDlg without save/close buttons
    # \param enable logical variable which dis-/enables mode
    def treeMode(self, enable=True):
        if enable:
            self.__dialog.ui.closeSaveFrame.hide()
            self.__datasource.tree = True
        else:
            self.__datasource.tree = False
            self.__dialog.ui.closeSaveFrame.show()

    #  creates GUI
    # \brief It calls setupUi and  connects signals and slots
    def createGUI(self):
        if not self.__dialog:
            self.createDialog()
        if self.__dialog and self.__dialog.ui \
                and not hasattr(self.__dialog.ui, "resetPushButton"):
            self.__dialog.ui.setupUi(self.__dialog)
            layout = QVBoxLayout()
            for ds in self.__dialog.wg.keys():
                self.__dialog.qwg[ds] = QWidget(self.__dialog)
                self.__dialog.wg[ds].setupUi(self.__dialog.qwg[ds])
                layout.addWidget(self.__dialog.qwg[ds])

            self.__dialog.ui.dsFrame.setLayout(layout)

        self.updateForm()
        self.__dialog.resize(460, 550)

        if not self.__datasource.tree:

            try:
                self.__dialog.ui.resetPushButton.clicked.disconnect(self.reset)
            except Exception:
                pass
            self.__dialog.ui.resetPushButton.clicked.connect(self.reset)
        else:
            try:
                self.__dialog.ui.resetPushButton.clicked.disconnect(
                    self.__dialog.reset)
            except Exception:
                pass
            self.__dialog.ui.resetPushButton.clicked.connect(
                self.__dialog.reset)
        self.__dialog.connectWidgets()
        self.__dialog.setFrames(self.__datasource.dataSourceType)

    # sets the form from the DOM node
    # \param node DOM node
    def setFromNode(self, node=None):
        if not self.__dialog:
            self.createDialog()
        if node:
            # defined in NodeDlg class
            self.__dialog.node = node
            if self.__dialog:
                self.__dialog.node = node
        if not self.__dialog.node \
                or not hasattr(self.__dialog.node, "attributes"):
            return
        attributeMap = self.__dialog.node.attributes()

        value = attributeMap.namedItem("type").nodeValue() \
            if attributeMap.contains("type") else ""
        self.__datasource.dataSourceType = unicode(value)

        if attributeMap.contains("name"):
            self.__datasource.dataSourceName = \
                attributeMap.namedItem("name").nodeValue()

        if value in self.__dialog.imp.keys():
            self.__dialog.imp[str(value)].setFromNode(self.__datasource)

        doc = self.__dialog.node.firstChildElement(str("doc"))
        text = DomTools.getText(doc)
        self.__datasource.doc = unicode(text).strip() if text else ""

    # accepts input text strings
    # \brief It copies the parameters and accept the self.__dialog
    def apply(self):
        if not self.__dialog:
            self.createDialog()

        self.__datasource.applied = False
        sourceType = unicode(self.__dialog.ui.typeComboBox.currentText())
        self.__datasource.dataSourceName = unicode(
            self.__dialog.ui.nameLineEdit.text())

        if sourceType in self.__dialog.imp.keys():
            self.__dialog.imp[sourceType].fromForm(self.__datasource)

        self.__datasource.dataSourceType = sourceType
        self.__datasource.doc = unicode(
            self.__dialog.ui.docTextEdit.toPlainText()).strip()

        index = QModelIndex()
        if hasattr(self.__dialog, "view") and self.__dialog.view \
                and self.__dialog.view.model():
            if hasattr(self.__dialog.view, "currentIndex"):
                index = self.__dialog.view.currentIndex()
                finalIndex = self.__dialog.view.model().createIndex(
                    index.row(), 2, index.parent().internalPointer())
                self.__dialog.view.expand(index)

        row = index.row()
        column = index.column()
        parent = index.parent()

        if self.__dialog.root:
            self.updateNode(index)

            if index.isValid():
                index = self.__dialog.view.model().index(row, column, parent)
                self.__dialog.view.setCurrentIndex(index)
                self.__dialog.view.expand(index)

            if hasattr(self.__dialog, "view") and self.__dialog.view \
                    and self.__dialog.view.model():
                self.__dialog.view.model().dataChanged.emit(
                    index.parent(), index.parent())
                if index.column() != 0:
                    index = self.__dialog.view.model().index(index.row(), 0,
                                                             index.parent())
                self.__dialog.view.model().dataChanged.emit(
                    index, finalIndex)
                self.__dialog.view.expand(index)

        if not self.__datasource.tree:
            self.createNodes()

        self.__datasource.applied = True

        return True

    def __createDOMNodes(self, root):
        newDs = root.createElement(str("datasource"))
        elem = newDs.toElement()
#        attributeMap = self.__datasource.newDs.attributes()
        elem.setAttribute(str("type"),
                          str(self.__datasource.dataSourceType))
        if self.__datasource.dataSourceName:
            elem.setAttribute(str("name"),
                              str(self.__datasource.dataSourceName))
        else:
            logger.info("name not defined")

        if self.__datasource.dataSourceType in self.__dialog.imp.keys():
            self.__dialog.imp[str(self.__datasource.dataSourceType)
                              ].createNodes(self.__datasource, root, elem)

        if(self.__datasource.doc):
            newDoc = root.createElement(str("doc"))
            newText = root.createTextNode(str(self.__datasource.doc))
            newDoc.appendChild(newText)
            elem.appendChild(newDoc)
        return elem

    # creates datasource node
    # \param external True if it should be create on a local DOM root,
    #        i.e. in component tree
    # \returns created DOM node
    def createNodes(self, external=False):
        if not self.__dialog:
            self.createDialog()
        if external:
            root = QDomDocument()
        else:
            if not self.__dialog.root or not self.__dialog.node:
                self.createHeader()
            root = self.__dialog.root

        elem = self.__createDOMNodes(root)

        if external and hasattr(self.__dialog.root, "importNode"):
            rootDs = self.__dialog.root.importNode(elem, True)
        else:
            rootDs = elem
        return rootDs

    # updates the Node
    # \brief It sets node from the self.__dialog variables
    def updateNode(self, index=QModelIndex()):
        if not self.__dialog:
            self.createDialog()

        newDs = self.createNodes(self.__datasource.tree)
        oldDs = self.__dialog.node

        if hasattr(index, "parent"):
            parent = index.parent()
        else:
            parent = QModelIndex()

        self.__dialog.node = self.__dialog.node.parentNode()
        if self.__datasource.tree:
            if self.__dialog.view is not None \
                    and self.__dialog.view.model() is not None:
                DomTools.replaceNode(oldDs, newDs, parent,
                                     self.__dialog.view.model())
        else:
            self.__dialog.node.replaceChild(newDs, oldDs)
        self.__dialog.node = newDs

    # reconnects save actions
    # \brief It reconnects the save action
    def reconnectSaveAction(self):
        if not self.__dialog:
            self.createDialog()
        if self.__datasource.externalSave:
            try:
                self.__dialog.ui.savePushButton.clicked.disconnect(
                    self.__datasource.externalSave)
            except Exception:
                pass
            self.__dialog.ui.savePushButton.clicked.connect(
                self.__datasource.externalSave)
            # self.__parent.disconnect(self.__dialog.ui.savePushButton,
            #                       SIGNAL("clicked()"),
            #                       self.__datasource.externalSave)
            # self.__parent.connect(self.__dialog.ui.savePushButton,
            #                       SIGNAL("clicked()"),
            #                       self.__datasource.externalSave)
        if self.__datasource.externalStore:
            try:
                self.__dialog.ui.storePushButton.clicked.disconnect(
                    self.__datasource.externalStore)
            except Exception:
                pass
            self.__dialog.ui.storePushButton.clicked.connect(
                self.__datasource.externalStore)
            # self.__parent.disconnect(self.__dialog.ui.storePushButton,
            #                          SIGNAL("clicked()"),
            #                          self.__datasource.externalStore)
            # self.__parent.connect(self.__dialog.ui.storePushButton,
            #                       SIGNAL("clicked()"),
            #                       self.__datasource.externalStore)
        if self.__datasource.externalClose:
            try:
                self.__dialog.ui.closePushButton.clicked.disconnect(
                    self.__datasource.externalClose)
            except Exception:
                pass
            self.__dialog.ui.closePushButton.clicked.connect(
                                     self.__datasource.externalClose)
            # self.__parent.disconnect(self.__dialog.ui.closePushButton,
            #                          SIGNAL("clicked()"),
            #                          self.__datasource.externalClose)
            # self.__parent.connect(self.__dialog.ui.closePushButton,
            #                       SIGNAL("clicked()"),
            #                       self.__datasource.externalClose)
        if self.__datasource.externalApply:
            try:
                self.__dialog.ui.applyPushButton.clicked.disconnect(
                    self.__datasource.externalApply)
            except Exception:
                pass
            self.__dialog.ui.applyPushButton.clicked.connect(
                self.__datasource.externalApply)
            # self.__parent.disconnect(self.__dialog.ui.applyPushButton,
            #                          SIGNAL("clicked()"),
            #                          self.__datasource.externalApply)
            # self.__parent.connect(self.__dialog.ui.applyPushButton,
            #                       SIGNAL("clicked()"),
            #                       self.__datasource.externalApply)

    # connects the save action and stores the apply action
    # \param externalApply apply action
    # \param externalSave save action
    # \param externalClose close action
    # \param externalStore store action
    def connectExternalActions(self, externalApply=None, externalSave=None,
                               externalClose=None, externalStore=None):
        if not self.__dialog:
            self.createDialog()
        if externalSave and self.__datasource.externalSave is None:
            try:
                self.__dialog.ui.savePushButton.clicked.disconnect(
                    externalSave)
            except Exception:
                pass
            self.__dialog.ui.savePushButton.clicked.connect(
                externalSave)
            # self.__parent.disconnect(self.__dialog.ui.savePushButton,
            #                          SIGNAL("clicked()"),
            #                          externalSave)
            # self.__parent.connect(self.__dialog.ui.savePushButton,
            #                       SIGNAL("clicked()"),
            #                       externalSave)
            self.__datasource.externalSave = externalSave
        if externalStore and self.__datasource.externalStore is None:
            try:
                self.__dialog.ui.storePushButton.clicked.disconnect(
                    externalStore)
            except Exception:
                pass
            self.__dialog.ui.storePushButton.clicked.connect(
                externalStore)
            # self.__parent.disconnect(self.__dialog.ui.storePushButton,
            #                          SIGNAL("clicked()"),
            #                          externalStore)
            # self.__parent.connect(self.__dialog.ui.storePushButton,
            #                       SIGNAL("clicked()"),
            #                       externalStore)
            self.__datasource.externalStore = externalStore
        if externalClose and self.__datasource.externalClose is None:
            try:
                self.__dialog.ui.closePushButton.clicked.disconnect(
                    externalClose)
            except Exception:
                pass
            self.__dialog.ui.closePushButton.clicked.connect(
                externalClose)
            # self.__parent.disconnect(self.__dialog.ui.closePushButton,
            #                          SIGNAL("clicked()"),
            #                          externalClose)
            # self.__parent.connect(self.__dialog.ui.closePushButton,
            #                       SIGNAL("clicked()"),
            #                       externalClose)
            self.__datasource.externalClose = externalClose
        if externalApply and self.__datasource.externalApply is None:
            try:
                self.__dialog.ui.applyPushButton.clicked.disconnect(
                    externalApply)
            except Exception:
                pass
            self.__dialog.ui.applyPushButton.clicked.connect(
                externalApply)
            # self.__parent.disconnect(self.__dialog.ui.applyPushButton,
            #                          SIGNAL("clicked()"),
            #                          externalApply)
            # self.__parent.connect(self.__dialog.ui.applyPushButton,
            #                       SIGNAL("clicked()"),
            #                       externalApply)
            self.__datasource.externalApply = externalApply

    # creates the new empty header
    # \brief It clean the DOM tree and put into it xml and definition nodes
    def createHeader(self):
        if not self.__dialog:
            self.createDialog()
        if hasattr(self.__dialog, "view") and self.__dialog.view:
            self.__dialog.view.setModel(None)
        self.__datasource.document = QDomDocument()
        # defined in NodeDlg class
        self.__dialog.root = self.__datasource.document
        processing = self.__dialog.root.createProcessingInstruction(
            "xml", "version='1.0'")
        self.__dialog.root.appendChild(processing)

        definition = self.__dialog.root.createElement(str("definition"))
        self.__dialog.root.appendChild(definition)
        self.__dialog.node = self.__dialog.root.createElement(
            str("datasource"))
        definition.appendChild(self.__dialog.node)
        return self.__dialog.node

    # copies the datasource to the clipboard
    # \brief It copies the current datasource to the clipboard
    def copyToClipboard(self):
        dsNode = self.createNodes(True)
        doc = QDomDocument()
        child = doc.importNode(dsNode, True)
        doc.appendChild(child)
        text = unicode(doc.toString(0))
        clipboard = QApplication.clipboard()
        clipboard.setText(text)

    # copies the datasource from the clipboard to the current datasource
    #       dialog
    # \return status True on success
    def copyFromClipboard(self):
        if not self.__dialog:
            self.createDialog()
        clipboard = QApplication.clipboard()
        text = unicode(clipboard.text())
        self.__datasource.document = QDomDocument()
        self.__dialog.root = self.__datasource.document
        if not self.__datasource.document.setContent(
                self.__datasource.repair(text)):
            raise ValueError("could not parse XML")
        else:
            if self.__dialog and hasattr(self.__dialog, "root"):
                self.__dialog.root = self.__datasource.document
                self.__dialog.node = DomTools.getFirstElement(
                    self.__datasource.document, "datasource")
        if not self.__dialog.node:
            return
        self.setFromNode(self.__dialog.node)

        return True


if __name__ == "__main__":
    pass



#!/usr/bin/env python
#   This file is part of nexdatas - Tango Server for NeXus data writer
#
#    Copyright (C) 2012-2017 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
# \package nxsconfigtool nexdatas
# \file ComponentItem.py
# dom item

""" DOM item for component model"""


# dialog defining a tag link
class ComponentItem(object):

    # constructor
    # \param node DOM node of item
    # \param parent patent instance
    def __init__(self, node, parent=None):
        # DOM node
        self.node = node
        # list with child items
        self.__childItems = []
        # the parent ComponentItem of the item
        self.parent = parent

    # provides indexs of given child
    # \param child
    # \returns child index
    def index(self, child):
        return self.__childItems.index(child)

    # provides a number of the current item
    # \returns a number of the current item
    def childNumber(self):
        if self.parent:
            return self.parent.index(self)
        return 0

    # provides the child item for the given list index
    # \param i child index
    # \returns requested child Item
    def child(self, i):
        size = len(self.__childItems)
        if i in range(size):
            return self.__childItems[i]
        if i >= 0 and i < self.node.childNodes().count():
            for j in range(size, i + 1):
                childNode = self.node.childNodes().item(j)
                childItem = ComponentItem(childNode, self)
                self.__childItems.append(childItem)
            return childItem

    # removes the given children from the child item list
    # \param position list index of the first child to remove
    # \param count number of children to remove
    # \returns if indices not out of range
    def removeChildren(self, position, count):
        if position < 0 or position + count > self.node.childNodes().count():
            return False

        for _ in range(count):
            if position < len(self.__childItems):
                self.__childItems.pop(position)

        return True

    # inserts the given children into the child item list
    # \param position list index of the first child to remove
    # \param count number of children to remove
    # \returns if indices not out of range
    def insertChildren(self, position, count):

        if position < 0 or position > self.node.childNodes().count():
            return False

        for i in range(position, position + count):
            if position <= len(self.__childItems):
                childNode = self.node.childNodes().item(i)
                childItem = ComponentItem(childNode, self)
                self.__childItems.insert(i, childItem)

        return True


if __name__ == "__main__":

    from PyQt5.QtXml import QDomNode

    # DOM node
    qdn = QDomNode()
    # instance of component item
    di = ComponentItem(qdn, None)
    di.child(0)



#!/usr/bin/env python
#   This file is part of nexdatas - Tango Server for NeXus data writer
#
#    Copyright (C) 2012-2017 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
# \package nxsconfigtool nexdatas
# \file AttributeDlg.py
# Attribute dialog class

""" attribute dialog """
import os

from PyQt5.QtWidgets import (QDialog, QDialogButtonBox, QMessageBox)
from PyQt5 import uic

# from .ui.ui_attributedlg import Ui_AttributeDlg
from .Errors import CharacterError

import logging
import sys
# message logger
logger = logging.getLogger("nxsdesigner")

_formclass, _baseclass = uic.loadUiType(
    os.path.join(os.path.dirname(os.path.abspath(__file__)),
                 "ui", "attributedlg.ui"))

if sys.version_info > (3,):
    unicode = str


# dialog defining a tag attribute
class AttributeDlg(QDialog):

    # constructor
    # \param parent patent instance
    def __init__(self, parent=None):
        super(AttributeDlg, self).__init__(parent)

        # attribute name
        self.name = u''
        # attribute value
        self.value = u''

        # user interface
        self.ui = _formclass()
        self.ui.setupUi(self)

        self.__updateUi()

        self.ui.nameLineEdit.textEdited.connect(self.__updateUi)
        # self.connect(self.ui.nameLineEdit,
        #              SIGNAL("textEdited(QString)"), self.__updateUi)

    # updates attribute user interface
    # \brief It sets enable or disable the OK button
    def __updateUi(self):
        enable = bool(self.ui.nameLineEdit.text())
        self.ui.buttonBox.button(QDialogButtonBox.Ok).setEnabled(enable)

    # accepts input text strings
    # \brief It copies the attribute name and value from lineEdit widgets
    #        and accept the dialog
    def accept(self):
        name = unicode(self.ui.nameLineEdit.text())

        try:
            if 1 in [c in name for c in '!"#$%&\'()*+,/;<=>?@[\\]^`{|}~']:
                raise CharacterError(
                    "Name contains one of forbidden characters")
            if len(name) == 0:
                raise CharacterError("Empty Name")
            if name[0] == '-':
                raise CharacterError("The first character of Name is '-'")

        except CharacterError as e:
            QMessageBox.warning(self, "Character Error", unicode(e))
            return
        self.name = name
        self.value = unicode(self.ui.valueLineEdit.text())

        QDialog.accept(self)


if __name__ == "__main__":
    import sys
    from PyQt5.QtGui import QApplication

    logging.basicConfig(level=logging.DEBUG)

    # Qt application
    app = QApplication(sys.argv)
    # attribute form
    form = AttributeDlg()
    form.show()
    app.exec_()

    if form.result():
        if form.name:
            logger.info("Attribute: %s = \'%s\'" % (form.name, form.value))



#!/usr/bin/env python
#   This file is part of nexdatas - Tango Server for NeXus data writer
#
#    Copyright (C) 2012-2017 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
# \package nxsconfigtool nexdatas
# \file ComponentList.py
# Data component list class


""" component list widget """


from .Component import Component
from .ElementList import ElementList

import logging
# message logger
logger = logging.getLogger("nxsdesigner")


# dialog defining a group tag
class ComponentList(ElementList):

    # constructor
    # \param directory element directory
    # \param parent patent instance
    def __init__(self, directory, parent=None):
        super(ComponentList, self).__init__(directory, parent)

        # show all attribures or only the type attribute
        self._allAttributes = False

        # widget title
        self.title = "Components"
        # element name
        self.name = "components"
        # class name
        self.clName = "Component"
        # extention
        self.extention = ".xml"
        # excluded extention
        self.disextention = "ds.xml"

    # switches between all attributes in the try or only type attribute
    # \param status all attributes are shown if True
    def viewAttributes(self, status=None):
        if status is None:
            return self._allAttributes
        self._allAttributes = True if status else False
        for k in self.elements.keys():
            if hasattr(self.elements[k], "instance") \
                    and self.elements[k].instance:
                self.elements[k].instance.viewAttributes(
                    self._allAttributes)

    # retrives element name from file name
    # \param fname filename
    # \returns element name
    @classmethod
    def nameFromFile(cls, fname):
        if fname[-4:] == '.xml':
            name = fname[:-4]
        else:
            name = fname
        return name

    # creates Element
    # \param name element name
    # \returns element instance
    def createElement(self, name):
        dlg = Component(self)
        dlg.directory = self.directory
        dlg.name = name
        dlg.createGUI()
        return dlg

    def dataSourceComponents(self, datasource):
        comps = set()
        for cp in self.elements.values():
            if cp and cp.name and cp.instance:
                if hasattr(cp.instance, "datasources"):
                    if datasource in cp.instance.datasources:
                        comps.add(cp.name)
        return list(comps)


if __name__ == "__main__":
    import sys
    from PyQt5.QtGui import QApplication

    logging.basicConfig(level=logging.DEBUG)

    # Qt application
    app = QApplication(sys.argv)
    # group form
    form = ComponentList("../components")
    form.createGUI()
    form.show()
    app.exec_()

    if form.elements:
        logger.info("Other components:")
        for mk in form.elements.keys():
            logger.info(" %s = '%s' " % (mk, form.elements[mk]))



#!/usr/bin/env python
#   This file is part of nexdatas - Tango Server for NeXus data writer
#
#    Copyright (C) 2012-2017 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
#

""" setup.py for NXS Component Designer """

import os
import sys
from setuptools import setup
from setuptools.command.build_py import build_py
from distutils.command.clean import clean
from distutils.util import get_platform
import shutil

try:
    from sphinx.setup_command import BuildDoc
except Exception:
    BuildDoc = None


#: package name
TOOL = "nxsconfigtool"
#: package instance
ITOOL = __import__(TOOL)


def read(fname):
    """ read the file

    :param fname: readme file name
    """
    return open(os.path.join(os.path.dirname(__file__), fname)).read()


#: ui directory
UIDIR = os.path.join(TOOL, "ui")
#: qrc directory
QRCDIR = os.path.join(TOOL, "qrc")
#: executable scripts
SCRIPTS = ['nxsdesigner']

needs_pytest = set(['test']).intersection(sys.argv)
pytest_runner = ['pytest-runner'] if needs_pytest else []

install_requires = [
    'nxstools',
    # 'pyqt5',
    # 'pytango',
]


class toolBuild(build_py):
    """ ui and qrc builder for python
    """

    @classmethod
    def makeqrc(cls, qfile, path):
        """  creates the python qrc files

        :param qfile: qrc file name
        :param path:  qrc file path
        """
        qrcfile = os.path.join(path, "%s.qrc" % qfile)
        rccfile = os.path.join(path, "%s.rcc" % qfile)

        compiled = os.system("rcc %s -o %s -binary" % (qrcfile, rccfile))
        if compiled == 0:
            print("Built: %s -> %s" % (qrcfile, rccfile))
        else:
            sys.stderr.write("Error: Cannot build  %s\n" % (rccfile))
            sys.stderr.flush()

    def run(self):
        """ runner

        :\brief: It is running during building
        """
        try:
            qfiles = [(qfile[:-4], QRCDIR) for qfile
                      in os.listdir(QRCDIR) if qfile.endswith('.qrc')]
            for qrc in qfiles:
                if not qrc[0] in (".", ".."):
                    self.makeqrc(qrc[0], qrc[1])
        except TypeError:
            sys.stderr.write("No .qrc files to build\n")
            sys.stderr.flush()

        if get_platform()[:3] == 'win':
            for script in SCRIPTS:
                shutil.copy(script, script + ".pyw")
        build_py.run(self)


class toolClean(clean):
    """ cleaner for python
    """

    def run(self):
        """ runner

        :brief: It is running during cleaning
        """

        cfiles = [os.path.join(TOOL, cfile) for cfile
                  in os.listdir("%s" % TOOL) if cfile.endswith('.pyc')]
        for fl in cfiles:
            os.remove(str(fl))

        if get_platform()[:3] == 'win':
            for script in SCRIPTS:
                if os.path.exists(script + ".pyw"):
                    os.remove(script + ".pyw")
        clean.run(self)


def get_scripts(scripts):
    """ provides windows names of python scripts

    :param scripts: list of script names
    """
    if get_platform()[:3] == 'win':
        return scripts + [sc + '.pyw' for sc in scripts]
    return scripts


release = ITOOL.__version__
version = ".".join(release.split(".")[:2])
name = "NXS Component Designer"

#: (:obj:`dict` <:obj:`str`, :obj:`list` <:obj:`str`> > ) package data
package_data = {
    'nxsconfigtool': ['ui/*.ui', 'qrc/*.rcc']
}


#: metadata for distutils
SETUPDATA = dict(
    name="nxsconfigtool",
    version=ITOOL.__version__,
    author="Jan Kotanski, Eugen Wintersberger , Halil Pasic",
    author_email="jankotan@gmail.com, eugen.wintersberger@gmail.com, "
    + "halil.pasic@gmail.com",
    maintainer="Jan Kotanski, Eugen Wintersberger , Halil Pasic",
    maintainer_email="jankotan@gmail.com, eugen.wintersberger@gmail.com, "
    "halil.pasic@gmail.com",
    description=("Configuration tool  for creating components"),
    license='GPLv3',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Science/Research',
        'Topic :: Scientific/Engineering :: Physics',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'License :: OSI Approved :: GNU General Public License v2 (GPLv2)',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],
    keywords="configuration writer Tango component nexus data",
    url="https://github.com/nexdatas/configtool/",
    install_requires=install_requires,
    platforms=("Linux", "Windows", "MacOS"),
    packages=[TOOL, QRCDIR],
    package_data=package_data,
    scripts=get_scripts(SCRIPTS),
    zip_safe=False,
    setup_requires=pytest_runner,
    tests_require=['pytest'],
    cmdclass={
        "build_py": toolBuild,
        "clean": toolClean,
        "build_sphinx": BuildDoc
    },
    command_options={
        'build_sphinx': {
            'project': ('setup.py', name),
            'version': ('setup.py', version),
            'release': ('setup.py', release)}},
    long_description=read('README.rst')
)


def main():
    """ the main function

    """
    setup(**SETUPDATA)


if __name__ == '__main__':
    main()



#!/usr/bin/env python
#   This file is part of nexdatas - Tango Server for NeXus data writer
#
#    Copyright (C) 2012-2017 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
# \package nxsconfigtool nexdatas
# \file HelpForm.py
# Detail help for Component Designer

""" help widget """


from PyQt5.QtCore import (QUrl, Qt)
from PyQt5.QtGui import (QKeySequence, QIcon)
from PyQt5.QtWidgets import (
    QAction, QApplication, QDialog,
    QLabel, QTextBrowser, QToolBar, QVBoxLayout)


# detail help
class HelpForm(QDialog):

    # constructor
    # \param page the starting html page
    # \param parent parent widget
    def __init__(self, page, parent=None):
        super(HelpForm, self).__init__(parent)

        self.setAttribute(Qt.WA_DeleteOnClose)
        self.setAttribute(Qt.WA_GroupLeader)

        # help tool bar
        self.toolBar = None
        # help text Browser
        self.textBrowser = None
        # main label of the help
        self.pageLabel = None

        self._page = page
        self.createGUI()
        self.createActions()

    #  creates GUI
    # \brief It create dialogs for help dialog
    def createGUI(self):
        # help tool bar
        self.toolBar = QToolBar(self)
        # help text Browser
        self.textBrowser = QTextBrowser(self)

        layout = QVBoxLayout(self)
        layout.addWidget(self.toolBar)
        layout.addWidget(self.textBrowser, 1)

        self.setLayout(layout)
        self.textBrowser.setSearchPaths([":/help"])
        self.textBrowser.setSource(QUrl(self._page))
        self.resize(660, 700)
        self.setWindowTitle("%s Help" % (
            QApplication.applicationName()))

    # creates actions
    # \brief It creates actions and sets the command pool and stack
    def createActions(self):

        backAction = QAction(QIcon(":/back.png"), "&Back", self)
        backAction.setShortcut(QKeySequence.Back)

        forwardAction = QAction(QIcon(":/forward.png"), "&Forward", self)
        forwardAction.setShortcut("Forward")

        homeAction = QAction(QIcon(":/home.png"), "&Home", self)
        homeAction.setShortcut("Home")

        # main label of the help
        self.pageLabel = QLabel(self)

        self.toolBar.addAction(backAction)
        self.toolBar.addAction(forwardAction)
        self.toolBar.addAction(homeAction)
        self.toolBar.addSeparator()
        self.toolBar.addWidget(self.pageLabel)

        try:
            backAction.triggered.disconnect(self.textBrowser.backward)
        except Exception:
            pass
        try:
            forwardAction.triggered.disconnect(self.textBrowser.forward)
        except Exception:
            pass
        try:
            homeAction.triggered.disconnect(self.textBrowser.home)
        except Exception:
            pass
        try:
            self.textBrowser.sourceChanged.disconnect(
                        self.updatePageTitle)
        except Exception:
            pass

        backAction.triggered.connect(self.textBrowser.backward)
        forwardAction.triggered.connect(self.textBrowser.forward)
        homeAction.triggered.connect(self.textBrowser.home)
        self.textBrowser.sourceChanged.connect(
                    self.updatePageTitle)

        self.updatePageTitle()

    # updates the title page
    # \brief It resets the pageLabel withg the document title
    def updatePageTitle(self):
        self.pageLabel.setText(
            "<p><b><i><font color='#0066ee' font size = 4>" +
            "&nbsp;&nbsp;" + self.textBrowser.documentTitle()
            + "</i></b></p></br>"
        )


if __name__ == "__main__":
    import sys
    # application instance
    app = QApplication(sys.argv)
    # help form
    form = HelpForm("index.html")
    form.show()
    app.exec_()
